from django.core.management.base import BaseCommand, CommandError

from ...utils import import_wordpress


class Command(BaseCommand):
    help = 'Import given XML files exported from WordPress'

    def add_arguments(self, parser):
        parser.add_argument('wordpress_xml', nargs='+', type=str)

    def handle(self, *args, **options):
        for wordpress_xml in options['wordpress_xml']:
            try:
                imported, errors = import_wordpress(wordpress_xml)
            except Exception as e:
                self.stderr.write(self.style.ERROR('Failed to import "{}": {}'.format(wordpress_xml, e)))
                raise CommandError(e)
            if errors:
                self.stderr.write(self.style.ERROR('Failed to import {} items'.format(wordpress_xml, e)))
            self.stdout.write(self.style.SUCCESS(
                'Successfully imported {} items from "{}"'.format(imported, wordpress_xml)
            ))



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2017-02-25 11:38
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cms_articles', '0007_plugins'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='article',
            options={'ordering': ('-order_date',), 'permissions': (('publish_article', 'Can publish article'),), 'verbose_name': 'article', 'verbose_name_plural': 'articles'},
        ),
        migrations.AddField(
            model_name='article',
            name='revision_id',
            field=models.PositiveIntegerField(default=0, editable=False),
        ),
        migrations.AlterField(
            model_name='articleplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='cms_articles_articleplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='articlescategoryplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='cms_articles_articlescategoryplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='articlesplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='cms_articles_articlesplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='articlesplugin',
            name='trees',
            field=models.ManyToManyField(blank=True, related_name='_articlesplugin_trees_+', to='cms.Page', verbose_name='trees'),
        ),
    ]



# -*- coding: utf-8 -*-
from datetime import timedelta

from cms.constants import PUBLISHER_STATE_DIRTY
from cms.utils.helpers import reversion_register
from django.db import models
from django.utils import timezone
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from filer.fields.image import FilerImageField

from djangocms_text_ckeditor.fields import HTMLField

from .article import Article
from .managers import TitleManager


@python_2_unicode_compatible
class Title(models.Model):
    # These are the fields whose values are compared when saving
    # a Title object to know if it has changed.
    editable_fields = [
        'title',
        'slug',
        'page_title',
        'menu_title',
        'meta_description',
    ]

    article = models.ForeignKey(Article, verbose_name=_('article'), related_name='title_set')
    language = models.CharField(_('language'), max_length=15, db_index=True)
    title = models.CharField(_('title'), max_length=255)
    description = HTMLField(_('description'), blank=True, default='',
                            help_text=_('The text displayed in an articles overview.'))
    page_title = models.CharField(_('page title'), max_length=255, blank=True, null=True,
                                  help_text=_('overwrite the title (html title tag)'))
    menu_title = models.CharField(_('menu title'), max_length=255, blank=True, null=True,
                                  help_text=_('overwrite the title in the articles overview'))
    meta_description = models.TextField(_('meta description'), max_length=155, blank=True, null=True,
                                        help_text=_('The text displayed in search engines.'))
    slug = models.SlugField(_('slug'), max_length=255, db_index=True, unique=False)
    creation_date = models.DateTimeField(_('creation date'), editable=False, default=timezone.now)
    image = FilerImageField(verbose_name=_('image'), related_name='+', blank=True, null=True)

    # Publisher fields
    published = models.BooleanField(_('is published'), blank=True, default=False)
    publisher_is_draft = models.BooleanField(default=True, editable=False, db_index=True)
    # This is misnamed - the one-to-one relation is populated on both ends
    publisher_public = models.OneToOneField('self', related_name='publisher_draft', null=True, editable=False)
    publisher_state = models.SmallIntegerField(default=0, editable=False, db_index=True)

    objects = TitleManager()

    class Meta:
        unique_together = (('language', 'article'),)
        app_label = 'cms_articles'

    def __str__(self):
        return u'%s (%s, %s)' % (self.title, self.slug, self.language)

    def is_dirty(self):
        return self.publisher_state == PUBLISHER_STATE_DIRTY

    def save_base(self, *args, **kwargs):
        '''Overridden save_base. If an instance is draft, and was changed, mark
        it as dirty.

        Dirty flag is used for changed nodes identification when publish method
        takes place. After current changes are published, state is set back to
        PUBLISHER_STATE_DEFAULT (in publish method).
        '''
        keep_state = getattr(self, '_publisher_keep_state', None)

        # Published articles should always have a publication date
        # if the article is published we set the publish date if not set yet.
        if self.article.publication_date is None and self.published:
            self.article.publication_date = timezone.now() - timedelta(seconds=5)

        if self.publisher_is_draft and not keep_state and self.is_new_dirty():
            self.publisher_state = PUBLISHER_STATE_DIRTY

        if keep_state:
            delattr(self, '_publisher_keep_state')
        return super(Title, self).save_base(*args, **kwargs)

    def is_new_dirty(self):
        if not self.pk:
            return True

        try:
            old_title = Title.objects.get(pk=self.pk)
        except Title.DoesNotExist:
            return True

        for field in self.editable_fields:
            old_val = getattr(old_title, field)
            new_val = getattr(self, field)
            if not old_val == new_val:
                return True
        return False


def _reversion():
    exclude_fields = ['publisher_is_draft', 'publisher_public', 'publisher_state']

    reversion_register(
        Title,
        exclude_fields=exclude_fields
    )


_reversion()



from cms.utils.i18n import get_language_tuple
from django import forms
from django.core.exceptions import ValidationError
from django.forms.utils import ErrorList
from django.utils.text import slugify
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _

from djangocms_text_ckeditor.fields import HTMLFormField

from ..conf import settings
from ..models import Article, Title
from ..utils import is_valid_article_slug


class ArticleForm(forms.ModelForm):
    language = forms.ChoiceField(label=_('Language'), choices=get_language_tuple(),
                                 help_text=_('The current language of the content fields.'))
    title = Title._meta.get_field('title').formfield()
    slug = Title._meta.get_field('slug').formfield()
    description = Title._meta.get_field('description').formfield()
    page_title = Title._meta.get_field('page_title').formfield()
    menu_title = Title._meta.get_field('menu_title').formfield()
    meta_description = Title._meta.get_field('meta_description').formfield()
    image = Title._meta.get_field('image').formfield()

    class Meta:
        model = Article
        fields = ['tree', 'template', 'login_required']

    def __init__(self, *args, **kwargs):
        super(ArticleForm, self).__init__(*args, **kwargs)
        self.fields['language'].widget = forms.HiddenInput()
        if self.fields['tree'].widget.choices.queryset.count() == 1:
            self.fields['tree'].initial = self.fields['tree'].widget.choices.queryset.first()
            self.fields['tree'].widget = forms.HiddenInput()

    def clean(self):
        cleaned_data = self.cleaned_data
        slug = cleaned_data.get('slug', '')

        article = self.instance
        lang = cleaned_data.get('language', None)
        # No language, can not go further, but validation failed already
        if not lang:
            return cleaned_data
        tree = self.cleaned_data.get('tree', None)
        if tree and not is_valid_article_slug(article, lang, slug):
            self._errors['slug'] = ErrorList([_('Another article with this slug already exists')])
            del cleaned_data['slug']
        return cleaned_data

    def clean_slug(self):
        slug = slugify(self.cleaned_data['slug'])
        if not slug:
            raise ValidationError(_('Slug must not be empty.'))
        return settings.CMS_ARTICLES_SLUG_FORMAT.format(
            now=self.instance.creation_date or now(),
            slug=slug)


class ArticleCreateForm(ArticleForm):
    content = HTMLFormField(label=_('Text'), help_text=_('Initial content of the article.'), required=False)


class PublicationDatesForm(forms.ModelForm):
    language = forms.ChoiceField(label=_("Language"), choices=get_language_tuple(),
                                 help_text=_('The current language of the content fields.'))

    def __init__(self, *args, **kwargs):
        super(PublicationDatesForm, self).__init__(*args, **kwargs)
        self.fields['language'].widget = forms.HiddenInput()

    class Meta:
        model = Article
        fields = ['publication_date', 'publication_end_date']



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-16 12:58
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import djangocms_text_ckeditor.fields
import filer.fields.image


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0001_initial'),
        ('cms_articles', '0002_remove_title_excerpt'),
    ]

    operations = [
        migrations.AddField(
            model_name='title',
            name='description',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', help_text='The text displayed in an articles overview.', verbose_name='description'),
        ),
        migrations.AddField(
            model_name='title',
            name='image',
            field=filer.fields.image.FilerImageField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='filer.Image', verbose_name='image'),
        ),
        migrations.AlterField(
            model_name='title',
            name='menu_title',
            field=models.CharField(blank=True, help_text='overwrite the title in the articles overview', max_length=255, null=True, verbose_name='menu title'),
        ),
        migrations.AlterField(
            model_name='title',
            name='meta_description',
            field=models.TextField(blank=True, help_text='The text displayed in search engines.', max_length=155, null=True, verbose_name='meta description'),
        ),
        migrations.AlterField(
            model_name='title',
            name='page_title',
            field=models.CharField(blank=True, help_text='overwrite the title (html title tag)', max_length=255, null=True, verbose_name='page title'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-20 20:00
from __future__ import unicode_literals

import cms.models.fields
from cms.models import Page
from cms_articles.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import filer.fields.folder


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('filer', '0002_auto_20150606_2003'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('cms_articles', '0004_categories'),
        ('cms', '0013_urlconfrevision'),
    ]

    operations = [
        migrations.CreateModel(
            name='Author',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('author_id', models.IntegerField(unique=True, verbose_name='author id')),
                ('login', models.CharField(max_length=255, verbose_name='login name')),
                ('email', models.EmailField(blank=True, max_length=254, null=True, verbose_name='email')),
                ('first_name', models.CharField(blank=True, max_length=255, null=True, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=255, null=True, verbose_name='last name')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'verbose_name': 'author',
                'verbose_name_plural': 'authors',
            },
        ),
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('term_id', models.IntegerField(unique=True, verbose_name='term id')),
                ('name', models.CharField(max_length=255, verbose_name='name')),
                ('slug', models.SlugField(verbose_name='slug')),
                ('parent', models.CharField(blank=True, max_length=255, null=True, verbose_name='parent slug')),
                ('cached_name', models.CharField(blank=True, max_length=512, null=True, verbose_name='name')),
                ('category', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='cms_articles.Category', verbose_name='articles category')),
            ],
            options={
                'verbose_name': 'category',
                'verbose_name_plural': 'categories',
            },
        ),
        migrations.CreateModel(
            name='Item',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.TextField(default='', verbose_name='title')),
                ('link', models.CharField(max_length=255, verbose_name='link')),
                ('pub_date', models.DateTimeField(verbose_name='publication date')),
                ('guid', models.CharField(max_length=255, verbose_name='url')),
                ('description', models.TextField(verbose_name='description')),
                ('content', models.TextField(verbose_name='content')),
                ('excerpt', models.TextField(verbose_name='excerpt')),
                ('post_id', models.IntegerField(unique=True, verbose_name='post id')),
                ('post_date', models.DateTimeField(verbose_name='post date')),
                ('post_name', models.CharField(max_length=255, verbose_name='post name')),
                ('status', models.CharField(max_length=20, verbose_name='status')),
                ('post_parent', models.IntegerField(verbose_name='parent post id')),
                ('post_type', models.CharField(max_length=20, verbose_name='type')),
                ('postmeta', models.TextField(verbose_name='metadata')),
                ('article', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='cms_articles.Article', verbose_name='imported article')),
                ('categories', models.ManyToManyField(blank=True, related_name='kategorie', to='import_wordpress.Category')),
                ('created_by', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='import_wordpress.Author', verbose_name='created by')),
                ('file', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='filer.File', verbose_name='imported file')),
                ('folder', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='filer.Folder', verbose_name='attachments folder')),
                ('page', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='cms.Page', verbose_name='imported page')),
            ],
            options={
                'verbose_name': 'item',
                'verbose_name_plural': 'items',
            },
        ),
        migrations.CreateModel(
            name='Options',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True, verbose_name='name')),
                ('language', models.CharField(help_text='The language of the content fields.', max_length=15, verbose_name='language')),
                ('article_template', models.CharField(choices=settings.CMS_ARTICLES_TEMPLATES, default=settings.CMS_ARTICLES_TEMPLATES[0][0], max_length=100, verbose_name='template')),
                ('article_slot', models.CharField(default=settings.CMS_ARTICLES_SLOT, help_text='The name of placeholder used to create content plugins in.', max_length=255, verbose_name='slot')),
                ('article_redirects', models.BooleanField(default=True, help_text='Create django redirects for each article from the old path to the new imported path', verbose_name='create redirects')),
                ('article_publish', models.BooleanField(default=False, help_text='Publish imported articles.', verbose_name='publish')),
                ('page_template', models.CharField(choices=Page.template_choices, default=Page.TEMPLATE_DEFAULT, max_length=100, verbose_name='template')),
                ('page_slot', models.CharField(default='content', help_text='The name of placeholder used to create content plugins in.', max_length=255, verbose_name='slot')),
                ('page_redirects', models.BooleanField(default=True, help_text='Create django redirects for each page from the old path to the new imported path', verbose_name='create redirects')),
                ('page_publish', models.BooleanField(default=False, help_text='Publish imported pages.', verbose_name='publish')),
                ('article_folder', filer.fields.folder.FilerFolderField(blank=True, help_text='Select folder for articles. Subfolder will be created for each article with attachments.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='filer.Folder', verbose_name='attachments folder')),
                ('article_tree', models.ForeignKey(help_text='All posts will be imported as articles in this tree.', on_delete=django.db.models.deletion.CASCADE, related_name='+', to='cms.Page', verbose_name='tree')),
                ('file_folder', filer.fields.folder.FilerFolderField(blank=True, help_text='Select folder for other attachments.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='filer.Folder', verbose_name='folder')),
                ('gallery_folder', filer.fields.folder.FilerFolderField(blank=True, help_text='Select folder for galleries. Subfolder will be created for each gallery.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='filer.Folder', verbose_name='folder')),
                ('page_folder', filer.fields.folder.FilerFolderField(blank=True, help_text='Select folder for pages. Subfolder will be created for each page with attachments.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='filer.Folder', verbose_name='attachments folder')),
                ('page_root', cms.models.fields.PageField(blank=True, help_text='All pages will be imported as sub-pages of this page.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='cms.Page', verbose_name='root')),
                ('slide_folder', filer.fields.folder.FilerFolderField(blank=True, help_text='Select folder for slides.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='filer.Folder', verbose_name='folder')),
            ],
            options={
                'verbose_name': 'options',
                'verbose_name_plural': 'options',
            },
        ),
    ]



from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool
from django.utils.translation import ugettext_lazy as _

from .urls import urlpatterns


class CMSArticlesApp(CMSApp):
    name = _('Articles tree')
    urls = [urlpatterns]
    app_name = 'cms_articles'


apphook_pool.register(CMSArticlesApp)



# -*- coding: utf-8 -*-
import operator
import warnings

from cms.exceptions import DuplicatePlaceholderWarning
from cms.utils import get_cms_setting
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.db.models.query_utils import Q
from django.template import (
    Context, NodeList, Template, TemplateSyntaxError, Variable,
)
from django.template.base import VariableNode
from django.template.loader import get_template
from django.template.loader_tags import BlockNode, ExtendsNode
from django.utils import six
from django.utils.encoding import force_text

try:
    from django.template.loader_tags import ConstantIncludeNode as IncludeNode
except ImportError:
    from django.template.loader_tags import IncludeNode

try:
    from sekizai.helpers import get_varname, is_variable_extend_node, engines
except ImportError:
    from sekizai.helpers import get_varname, is_variable_extend_node
    engines = None


def _get_nodelist(tpl):
    if hasattr(tpl, 'template'):
        return tpl.template.nodelist
    else:
        return tpl.nodelist


def get_context():
    if engines is not None:
        context = Context()
        context.template = Template('')
        return context
    else:
        return {}


def get_placeholder_conf(setting, placeholder, template=None, default=None):
    """
    Returns the placeholder configuration for a given setting. The key would for
    example be 'plugins' or 'name'.

    If a template is given, it will try
    CMS_PLACEHOLDER_CONF['template placeholder'] and
    CMS_PLACEHOLDER_CONF['placeholder'], if no template is given only the latter
    is checked.
    """
    if placeholder:
        keys = []
        if template:
            keys.append("%s %s" % (template, placeholder))
        keys.append(placeholder)
        for key in keys:
            conf = get_cms_setting('PLACEHOLDER_CONF').get(key)
            if not conf:
                continue
            value = conf.get(setting)
            if value is not None:
                return value
            inherit = conf.get('inherit')
            if inherit:
                if ' ' in inherit:
                    inherit = inherit.split(' ')
                else:
                    inherit = (None, inherit,)
                value = get_placeholder_conf(setting, inherit[1], inherit[0], default)
                if value is not None:
                    return value
    return default


def get_toolbar_plugin_struct(plugins_list, slot, page, parent=None):
    """
    Return the list of plugins to render in the toolbar.
    The dictionary contains the label, the classname and the module for the
    plugin.
    Names and modules can be defined on a per-placeholder basis using
    'plugin_modules' and 'plugin_labels' attributes in CMS_PLACEHOLDER_CONF

    :param plugins_list: list of plugins valid for the placeholder
    :param slot: placeholder slot name
    :param page: the page
    :param parent: parent plugin class, if any
    :return: list of dictionaries
    """
    template = None
    if page:
        template = page.template
    main_list = []
    for plugin in plugins_list:
        allowed_parents = plugin().get_parent_classes(slot, page)
        if parent:
            # skip to the next if this plugin is not allowed to be a child
            # of the parent
            if allowed_parents and parent.__name__ not in allowed_parents:
                continue
        else:
            if allowed_parents:
                continue
        modules = get_placeholder_conf("plugin_modules", slot, template, default={})
        names = get_placeholder_conf("plugin_labels", slot, template, default={})
        main_list.append({'value': plugin.value,
                          'name': force_text(names.get(plugin.value, plugin.name)),
                          'module': force_text(modules.get(plugin.value, plugin.module))})
    return sorted(main_list, key=operator.itemgetter("module"))


def validate_placeholder_name(name):
    if not isinstance(name, six.string_types):
        raise ImproperlyConfigured("Placeholder identifier names need to be of type string. ")

    if not all(ord(char) < 128 for char in name):
        raise ImproperlyConfigured("Placeholder identifiers names may not "
                                   "contain non-ascii characters. If you wish your placeholder "
                                   "identifiers to contain non-ascii characters when displayed to "
                                   "users, please use the CMS_PLACEHOLDER_CONF setting with the 'name' "
                                   "key to specify a verbose name.")


class PlaceholderNoAction(object):
    can_copy = False

    def copy(self, **kwargs):
        return False

    def get_copy_languages(self, **kwargs):
        return []


class MLNGPlaceholderActions(PlaceholderNoAction):
    can_copy = True

    def copy(self, target_placeholder, source_language, fieldname, model, target_language, **kwargs):
        from cms.utils.copy_plugins import copy_plugins_to
        trgt = model.objects.get(**{fieldname: target_placeholder})
        src = model.objects.get(master=trgt.master, language_code=source_language)

        source_placeholder = getattr(src, fieldname, None)
        if not source_placeholder:
            return False
        return copy_plugins_to(source_placeholder.get_plugins_list(),
                               target_placeholder, target_language)

    def get_copy_languages(self, placeholder, model, fieldname, **kwargs):
        manager = model.objects
        src = manager.get(**{fieldname: placeholder})
        query = Q(master=src.master)
        query &= Q(**{'%s__cmsplugin__isnull' % fieldname: False})
        query &= ~Q(pk=src.pk)

        language_codes = manager.filter(query).values_list('language_code', flat=True).distinct()
        return [(lc, dict(settings.LANGUAGES)[lc]) for lc in language_codes]


def restore_sekizai_context(context, changes):
    varname = get_varname()
    sekizai_container = context.get(varname)
    for key, values in changes.items():
        sekizai_namespace = sekizai_container[key]
        for value in values:
            sekizai_namespace.append(value)


def _scan_placeholders(nodelist, current_block=None, ignore_blocks=None):
    from ..templatetags.cms_articles import ArticlePlaceholder

    placeholders = []
    if ignore_blocks is None:
        # List of BlockNode instances to ignore.
        # This is important to avoid processing overriden block nodes.
        ignore_blocks = []

    for node in nodelist:
        # check if this is a placeholder first
        if isinstance(node, ArticlePlaceholder):
            placeholders.append(node.get_name())
        elif isinstance(node, IncludeNode):
            # if there's an error in the to-be-included template, node.template becomes None
            if node.template:
                # This is required for Django 1.7 but works on older version too
                # Check if it quacks like a template object, if not
                # presume is a template path and get the object out of it
                if not callable(getattr(node.template, 'render', None)):
                    # If it's a variable there is no way to expand it at this stage so we
                    # need to skip it
                    if isinstance(node.template.var, Variable):
                        continue
                    else:
                        template = get_template(node.template.var)
                else:
                    template = node.template
                placeholders += _scan_placeholders(_get_nodelist(template), current_block)
        # handle {% extends ... %} tags
        elif isinstance(node, ExtendsNode):
            placeholders += _extend_nodelist(node)
        # in block nodes we have to scan for super blocks
        elif isinstance(node, VariableNode) and current_block:
            if node.filter_expression.token == 'block.super':
                if not hasattr(current_block.super, 'nodelist'):
                    raise TemplateSyntaxError("Cannot render block.super for blocks without a parent.")
                placeholders += _scan_placeholders(_get_nodelist(current_block.super), current_block.super)
        # ignore nested blocks which are already handled
        elif isinstance(node, BlockNode) and node.name in ignore_blocks:
            continue
        # if the node has the newly introduced 'child_nodelists' attribute, scan
        # those attributes for nodelists and recurse them
        elif hasattr(node, 'child_nodelists'):
            for nodelist_name in node.child_nodelists:
                if hasattr(node, nodelist_name):
                    subnodelist = getattr(node, nodelist_name)
                    if isinstance(subnodelist, NodeList):
                        if isinstance(node, BlockNode):
                            current_block = node
                        placeholders += _scan_placeholders(subnodelist, current_block, ignore_blocks)
        # else just scan the node for nodelist instance attributes
        else:
            for attr in dir(node):
                obj = getattr(node, attr)
                if isinstance(obj, NodeList):
                    if isinstance(node, BlockNode):
                        current_block = node
                    placeholders += _scan_placeholders(obj, current_block, ignore_blocks)
    return placeholders


def get_placeholders(template):
    compiled_template = get_template(template)
    placeholders = _scan_placeholders(_get_nodelist(compiled_template))
    clean_placeholders = []
    for placeholder in placeholders:
        if placeholder in clean_placeholders:
            warnings.warn("Duplicate {{% placeholder \"{0}\" %}} "
                          "in template {1}."
                          .format(placeholder, template, placeholder),
                          DuplicatePlaceholderWarning)
        else:
            validate_placeholder_name(placeholder)
            clean_placeholders.append(placeholder)
    return clean_placeholders


def _extend_nodelist(extend_node):
    """
    Returns a list of placeholders found in the parent template(s) of this
    ExtendsNode
    """
    # we don't support variable extensions
    if is_variable_extend_node(extend_node):
        return []
        # This is a dictionary mapping all BlockNode instances found in the template that contains extend_node
    blocks = dict(extend_node.blocks)
    _extend_blocks(extend_node, blocks)
    placeholders = []

    for block in blocks.values():
        placeholders += _scan_placeholders(_get_nodelist(block), block, blocks.keys())

    # Scan topmost template for placeholder outside of blocks
    parent_template = _find_topmost_template(extend_node)
    placeholders += _scan_placeholders(_get_nodelist(parent_template), None, blocks.keys())
    return placeholders


def _find_topmost_template(extend_node):
    parent_template = extend_node.get_parent(get_context())
    for node in _get_nodelist(parent_template).get_nodes_by_type(ExtendsNode):
        # Their can only be one extend block in a template, otherwise django raises an exception
        return _find_topmost_template(node)
        # No ExtendsNode
    return extend_node.get_parent(get_context())


def _extend_blocks(extend_node, blocks):
    """
    Extends the dictionary `blocks` with *new* blocks in the parent node (recursive)
    """
    # we don't support variable extensions
    if is_variable_extend_node(extend_node):
        return
    parent = extend_node.get_parent(get_context())
    # Search for new blocks
    for node in _get_nodelist(parent).get_nodes_by_type(BlockNode):
        if node.name not in blocks:
            blocks[node.name] = node
        else:
            # set this node as the super node (for {{ block.super }})
            block = blocks[node.name]
            seen_supers = []
            while hasattr(block.super, 'nodelist') and block.super not in seen_supers:
                seen_supers.append(block.super)
                block = block.super
            block.super = node
        # search for further ExtendsNodes
    for node in _get_nodelist(parent).get_nodes_by_type(ExtendsNode):
        _extend_blocks(node, blocks)
        break



from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from django.core.paginator import EmptyPage, PageNotAnInteger, Paginator
from django.utils.translation import ugettext as _

from .archive import Archive
from .conf import settings
from .models import ArticlePlugin, ArticlesCategoryPlugin, ArticlesPlugin


class ArticlePlugin(CMSPluginBase):
    module = _('Articles')
    name = _('Article')
    model = ArticlePlugin
    cache = False
    text_enabled = True
    raw_id_fields = ['article']

    def render(self, context, instance, placeholder):
        context.update({
            'plugin': instance,
            'article': instance.get_article(context),
            'placeholder': placeholder,
        })
        return context

    def get_render_template(self, context, instance, placeholder):
        return 'cms_articles/article/%s.html' % instance.template


plugin_pool.register_plugin(ArticlePlugin)


class ArticlesPlugin(CMSPluginBase):
    module = _('Articles')
    name = _('Articles')
    model = ArticlesPlugin
    cache = False
    text_enabled = True

    def render(self, context, instance, placeholder):
        # get articles based on plugin settings
        articles = instance.get_articles(context)

        # provide archive
        archive = Archive(articles, context['request'])

        # filter articles based on query
        articles = archive.filter_articles()

        # paginate articles
        paginator = Paginator(articles, instance.number)
        try:
            articles = paginator.page(context['request'].GET.get(settings.CMS_ARTICLES_PAGE_FIELD, 1))
        except PageNotAnInteger:
            # If page is not an integer, deliver first page.
            articles = paginator.page(1)
        except EmptyPage:
            # If page is out of range (e.g. 9999), deliver last page of results.
            articles = paginator.page(paginator.num_pages)
        articles.page_field = settings.CMS_ARTICLES_PAGE_FIELD

        context.update({
            'plugin': instance,
            'archive': archive,
            'articles': articles,
            'placeholder': placeholder,
        })
        return context

    def get_render_template(self, context, instance, placeholder):
        return 'cms_articles/articles/%s.html' % instance.template


plugin_pool.register_plugin(ArticlesPlugin)


class ArticlesCategoryPlugin(ArticlesPlugin):
    name = _('Articles category')
    model = ArticlesCategoryPlugin


plugin_pool.register_plugin(ArticlesCategoryPlugin)



from cms.models import CMSPlugin
from django.db.models import Q
from django.dispatch.dispatcher import receiver
from django.utils import timezone

from aldryn_search.helpers import get_plugin_index_data
from aldryn_search.signals import add_to_index, remove_from_index
from aldryn_search.utils import clean_join, get_index_base

from .conf import settings
from .models import Title
from .signals import post_publish_article, post_unpublish_article


class TitleIndex(get_index_base()):
    index_title = True

    object_actions = ('publish', 'unpublish')
    haystack_use_for_indexing = settings.CMS_ARTICLES_USE_HAYSTACK

    def prepare_pub_date(self, obj):
        return obj.article.publication_date

    def prepare_login_required(self, obj):
        return obj.article.login_required

    def prepare_site_id(self, obj):
        return obj.article.tree.site_id

    def get_language(self, obj):
        return obj.language

    def get_url(self, obj):
        return obj.article.get_absolute_url()

    def get_title(self, obj):
        return obj.title

    def get_description(self, obj):
        return obj.meta_description or None

    def get_plugin_queryset(self, language):
        queryset = CMSPlugin.objects.filter(language=language)
        return queryset

    def get_article_placeholders(self, article):
        """
        In the project settings set up the variable

        CMS_ARTICLES_PLACEHOLDERS_SEARCH_LIST = {
            'include': [ 'slot1', 'slot2', etc. ],
            'exclude': [ 'slot3', 'slot4', etc. ],
        }

        or leave it empty

        CMS_ARTICLES_PLACEHOLDERS_SEARCH_LIST = {}
        """
        placeholders_search_list = getattr(settings, 'CMS_ARTICLES_PLACEHOLDERS_SEARCH_LIST', {})

        included = placeholders_search_list.get('include', [])
        excluded = placeholders_search_list.get('exclude', [])
        diff = set(included) - set(excluded)
        if diff:
            return article.placeholders.filter(slot__in=diff)
        elif excluded:
            return article.placeholders.exclude(slot__in=excluded)
        else:
            return article.placeholders.all()

    def get_search_data(self, obj, language, request):
        current_article = obj.article
        placeholders = self.get_article_placeholders(current_article)
        plugins = self.get_plugin_queryset(language).filter(placeholder__in=placeholders)
        text_bits = []

        for base_plugin in plugins:
            plugin_text_content = self.get_plugin_search_text(base_plugin, request)
            text_bits.append(plugin_text_content)

        article_meta_description = current_article.get_meta_description(fallback=False, language=language)

        if article_meta_description:
            text_bits.append(article_meta_description)

        article_meta_keywords = getattr(current_article, 'get_meta_keywords', None)

        if callable(article_meta_keywords):
            text_bits.append(article_meta_keywords())

        return clean_join(' ', text_bits)

    def get_plugin_search_text(self, base_plugin, request):
        plugin_content_bits = get_plugin_index_data(base_plugin, request)
        return clean_join(' ', plugin_content_bits)

    def get_model(self):
        return Title

    def get_index_queryset(self, language):
        queryset = Title.objects.public().filter(
            Q(article__publication_date__lt=timezone.now()) | Q(article__publication_date__isnull=True),
            Q(article__publication_end_date__gte=timezone.now()) | Q(article__publication_end_date__isnull=True),
            language=language
        ).select_related('article').distinct()
        return queryset

    def should_update(self, instance, **kwargs):
        # We use the action flag to prevent
        # updating the cms article on save.
        return kwargs.get('object_action') in self.object_actions


@receiver(post_publish_article, dispatch_uid='publish_cms_article')
def publish_cms_article(sender, instance, language, **kwargs):
    title = instance.publisher_public.get_title_obj(language)
    print('##################### publish_cms_article', title)
    add_to_index.send(sender=Title, instance=title, object_action='publish')


@receiver(post_unpublish_article, dispatch_uid='unpublish_cms_article')
def unpublish_cms_article(sender, instance, language, **kwargs):
    title = instance.publisher_public.get_title_obj(language)
    print('##################### unpublish_cms_article', title)
    remove_from_index.send(sender=Title, instance=title, object_action='unpublish')



import calendar
from datetime import date

from django.utils.functional import cached_property

from .conf import settings


class Archive:
    def __init__(self, articles, request):
        self.articles = articles
        self.request = request
        self.year = self.month = self.day = None
        try:
            self.year = int(request.GET[settings.CMS_ARTICLES_YEAR_FIELD])
            self.month = int(request.GET[settings.CMS_ARTICLES_MONTH_FIELD])
            self.day = int(request.GET[settings.CMS_ARTICLES_DAY_FIELD])
        except:
            pass

    def filter_articles(self):
        articles = self.articles
        if self.year:
            articles = articles.filter(order_date__year=self.year)
            if self.month:
                articles = articles.filter(order_date__month=self.month)
                if self.day:
                    articles = articles.filter(order_date__day=self.day)
        return articles

    @cached_property
    def last(self):
        try:
            return self.articles.last().order_date
        except AttributeError:
            return date.today()

    def years(self):
        for year in range(date.today().year, self.last.year - 1, -1):
            yield YearArchive(year, self)

    @cached_property
    def date(self):
        if self.year:
            return date(self.year, self.month or 1, self.day or 1)
        else:
            return None


class YearArchive:
    def __init__(self, year, archive):
        self.year = year
        self.archive = archive
        self.articles = archive.articles.filter(order_date__year=year)
        self.active = archive.year == year

    def months(self):
        if self.year == date.today().year:
            first = date.today().month
        else:
            first = 12
        if self.year == self.archive.last.year:
            last = self.archive.last.month
        else:
            last = 1
        for month in range(first, last, -1):
            yield MonthArchive(month, self)

    @cached_property
    def date(self):
        return date(self.year, 1, 1)

    @cached_property
    def url(self):
        return '{path}?{y}={year}'.format(
            path=self.archive.request.path,
            y=settings.CMS_ARTICLES_YEAR_FIELD,
            year=self.year,
        )


class MonthArchive:
    def __init__(self, month, year_archive):
        self.month = month
        self.year_archive = year_archive
        self.articles = year_archive.articles.filter(order_date__month=month)
        self.active = year_archive.archive.month == month

    def days(self):
        if self.year == date.today().year and self.month == date.today().month:
            first = date.today().day
        else:
            first = calendar.monthrange(self.year_archive.year, self.month)[1]
        if self.year == self.year_archive.archive.last.year and self.month == self.year_archive.archive.last.month:
            last = self.year_archive.archive.last.day
        else:
            last = 1
        for day in range(first, last, -1):
            yield DayArchive(day, self)

    @cached_property
    def date(self):
        return date(self.year_archive.year, self.month, 1)

    @cached_property
    def url(self):
        return '{path}?{y}={year}&{m}={month}'.format(
            path=self.year_archive.archive.request.path,
            y=settings.CMS_ARTICLES_YEAR_FIELD,
            m=settings.CMS_ARTICLES_MONTH_FIELD,
            year=self.year_archive.year,
            month=self.month,
        )


class DayArchive:
    def __init__(self, day, month_archive):
        self.day = day
        self.month_archive = month_archive
        self.articles = month_archive.articles.filter(order_date__day=day)
        self.active = month_archive.year_archive.archive.day == day

    @cached_property
    def date(self):
        return date(self.month_archive.year_archive.year, self.month_archive.month, self.day)

    @cached_property
    def url(self):
        return '{path}?{y}={year}&{m}={month}&{d}={day}'.format(
            path=self.month_archive.year_archive.archive.request.path,
            y=settings.CMS_ARTICLES_YEAR_FIELD,
            m=settings.CMS_ARTICLES_MONTH_FIELD,
            d=settings.CMS_ARTICLES_DAY_FIELD,
            year=self.month_archive.year_archive.year,
            month=self.month_archive.month,
            day=self.day,
        )



from django.contrib import admin

from ..models import Category


class CategoryAdmin(admin.ModelAdmin):
    search_fields = ('page__title_set__slug', 'page__title_set__title')


admin.site.register(Category, CategoryAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-16 14:11
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion

from cms_articles.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0001_initial'),
        ('cms_articles', '0003_description_image'),
    ]

    operations = [
        migrations.RenameField(
            model_name='article',
            old_name='category',
            new_name='tree',
        ),
        migrations.RenameField(
            model_name='articlesplugin',
            old_name='category',
            new_name='tree',
        ),
        migrations.AlterField(
            model_name='article',
            name='tree',
            field=models.ForeignKey(help_text='The page the article is accessible at.', on_delete=django.db.models.deletion.CASCADE, related_name='cms_articles', to='cms.Page', verbose_name='tree'),
        ),
        migrations.AlterField(
            model_name='articlesplugin',
            name='tree',
            field=models.ForeignKey(blank=True, help_text='Keep empty to show articles from current page, if current page is a tree.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='cms.Page', verbose_name='tree'),
        ),
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('page', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='cms_articles_category', to='cms.Page', verbose_name='page')),
            ],
            options={
                'verbose_name': 'category',
                'verbose_name_plural': 'categories',
            },
        ),
        migrations.AddField(
            model_name='article',
            name='categories',
            field=models.ManyToManyField(blank=True, related_name='articles', to='cms_articles.Category', verbose_name='categories'),
        ),
        migrations.AddField(
            model_name='articlesplugin',
            name='categories',
            field=models.ManyToManyField(blank=True, related_name='_articlesplugin_categories_+', to='cms_articles.Category', verbose_name='categories'),
        ),
        migrations.CreateModel(
            name='ArticlesCategoryPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('number', models.IntegerField(default=3, verbose_name='Number of last articles')),
                ('template', models.CharField(choices=settings.CMS_ARTICLES_PLUGIN_ARTICLES_TEMPLATES, default=settings.CMS_ARTICLES_PLUGIN_ARTICLES_TEMPLATES[0][0], help_text='The template used to render plugin.', max_length=100, verbose_name='Template')),
                ('subcategories', models.BooleanField(default=False, help_text='Check, if you want to include articles from sub-categories of this category.', verbose_name='include sub-categories')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



from json import loads

from cms.api import create_page
from cms.models import Page
from cms.models.fields import PageField
from cms_articles.api import add_content, create_article, publish_article
from cms_articles.conf import settings
from django.core.files import File as DjangoFile
from django.core.files.temp import NamedTemporaryFile
from django.db import models
from django.utils.encoding import force_bytes, python_2_unicode_compatible
from django.utils.functional import cached_property
from django.utils.text import slugify
from django.utils.translation import ugettext_lazy as _
from filer.fields.folder import FilerFolderField
from filer.management.commands.import_files import FileImporter
from filer.models import File, Folder

from .utils import create_redirect

try:
    from urllib.request import urlopen
except ImportError:
    from urllib2 import urlopen


@python_2_unicode_compatible
class Author(models.Model):
    author_id = models.IntegerField(_('author id'), unique=True)
    login = models.CharField(_('login name'), max_length=255)
    email = models.EmailField(_('email'), blank=True, null=True)
    first_name = models.CharField(_('first name'), max_length=255, blank=True, null=True)
    last_name = models.CharField(_('last name'), max_length=255, blank=True, null=True)
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, verbose_name=_('user'),
        related_name='+', on_delete=models.SET_NULL, blank=True, null=True)

    def __str__(self):
        return '{}'.format(self.login)

    class Meta:
        verbose_name = _('author')
        verbose_name_plural = _('authors')


@python_2_unicode_compatible
class Category(models.Model):
    term_id = models.IntegerField(_('term id'), unique=True)
    name = models.CharField(_('name'), max_length=255)
    slug = models.SlugField(_('slug'))
    parent = models.CharField(_('parent slug'), max_length=255, blank=True, null=True)
    cached_name = models.CharField(_('name'), max_length=512, blank=True, null=True)
    category = models.ForeignKey(
        'cms_articles.Category', verbose_name=_('articles category'),
        related_name='+', on_delete=models.SET_NULL, blank=True, null=True)

    def __str__(self):
        try:
            parent = Category.objects.get(slug=self.parent)
        except Category.DoesNotExist:
            parent = None
        if parent:
            name = '{} / {}'.format(parent.name, self.name)
        else:
            name = '{}'.format(self.name)
        if name != self.cached_name:
            self.cached_name = name
            self.save()
        return name

    class Meta:
        verbose_name = _('category')
        verbose_name_plural = _('categories')


@python_2_unicode_compatible
class Item(models.Model):
    title = models.TextField(_('title'), default='')
    link = models.CharField(_('link'), max_length=255)
    pub_date = models.DateTimeField(_('publication date'))
    created_by = models.ForeignKey(Author, verbose_name=_('created by'))
    guid = models.CharField(_('url'), max_length=255)
    description = models.TextField(_('description'))
    content = models.TextField(_('content'))
    excerpt = models.TextField(_('excerpt'))
    post_id = models.IntegerField(_('post id'), unique=True)
    post_date = models.DateTimeField(_('post date'))
    post_name = models.CharField(_('post name'), max_length=255)
    status = models.CharField(_('status'), max_length=20)
    post_parent = models.IntegerField(_('parent post id'))
    post_type = models.CharField(_('type'), max_length=20)
    categories = models.ManyToManyField(Category, _('categories'), blank=True)
    postmeta = models.TextField(_('metadata'))
    article = models.OneToOneField(
        'cms_articles.Article', verbose_name=_('imported article'),
        related_name='+', on_delete=models.SET_NULL, blank=True, null=True)
    page = models.OneToOneField(
        'cms.Page', verbose_name=_('imported page'),
        related_name='+', on_delete=models.SET_NULL, blank=True, null=True)
    file = models.OneToOneField(
        File, verbose_name=_('imported file'),
        related_name='+', on_delete=models.SET_NULL, blank=True, null=True)
    folder = models.ForeignKey(
        Folder, verbose_name=_('attachments folder'),
        related_name='+', on_delete=models.SET_NULL, blank=True, null=True)

    def __str__(self):
        return '{}'.format(self.title)

    class Meta:
        verbose_name = _('item')
        verbose_name_plural = _('items')

    @property
    def children(self):
        return Item.objects.filter(post_parent=self.post_id)

    @cached_property
    def parent(self):
        if self.post_parent:
            try:
                return Item.objects.get(post_id=self.post_parent)
            except:
                pass
        return None

    @cached_property
    def meta(self):
        return loads(self.postmeta)

    def cms_import(self, options):
        obj = None
        if self.post_type == 'post':
            obj = self.get_or_import_article(options)
        elif self.post_type == 'page':
            obj = self.get_or_import_page(options)
        elif self.post_type == 'attachment':
            obj = self.get_or_import_file(options)
        # also import children
        for child in self.children.all():
            child.cms_import(options)
        return obj

    def get_or_import_article(self, options):
        assert self.post_type == 'post'
        if self.article:
            return self.article
        # import thumbnail
        image = None
        if '_thumbnail_id' in self.meta:
            image_item = Item.objects.get(post_id=int(self.meta['_thumbnail_id']))
            image = image_item.get_or_import_file(options)
        self.article = create_article(
            tree=options.article_tree,
            template=options.article_template,
            title=self.title,
            language=options.language,
            description=self.excerpt,
            created_by=self.created_by.user or self.created_by.login,
            image=image,
            publicationdate=self.pub_date,
            categories=[c.category for c in self.categories.exclude(category=None)],
        )
        self.article.creation_date = self.post_date
        self.article.save()
        content = '\n'.join('<p>{}</p>'.format(p) for p in self.content.split('\n\n'))
        add_content(self.article, language=options.language, slot=options.article_slot, content=content)
        if options.article_publish:
            self.article = publish_article(
                article=self.article,
                language=options.language,
                changed_by=self.created_by.user or self.created_by.login,
            )
            public = self.article.get_public_object()
            public.creation_date = self.pub_date
            public.save()
        if options.article_redirects:
            create_redirect(self.link, self.article.get_absolute_url())
        self.save()
        return self.article

    def get_or_import_page(self, options):
        assert self.post_type == 'page'
        if self.page:
            return self.page
        # import parent page first
        if self.parent:
            parent = self.parent.get_or_import_page(options)
        else:
            parent = options.page_root
        # get valid slug
        slug = self.post_name or slugify(self.title)
        assert slug
        # handle existing page
        self.page = Page.objects.filter(parent=parent, title_set__slug=slug).first()
        if self.page:
            self.save()
            return self.page
        # create new page
        self.page = create_page(
            template=options.page_template,
            language=options.language,
            title=self.title,
            slug=slug,
            meta_description=None,
            created_by=self.created_by.user or self.created_by.login,
            parent=parent,
            publication_date=self.pub_date,
        )
        self.page.creation_date = self.post_date
        self.page.save()
        content = '\n'.join('<p>{}</p>'.format(p) for p in self.content.split('\n\n'))
        add_content(self.page, language=options.language, slot=options.page_slot, content=content)
        if options.page_publish:
            self.page.publish(options.language)
            public = self.page.get_public_object()
            public.creation_date = self.pub_date
            public.save()
        if options.page_redirects:
            create_redirect(self.link, self.page.get_absolute_url())
        self.save()
        return self.page

    def get_or_import_file(self, options):
        assert self.post_type == 'attachment'
        if self.file:
            return self.file
        # download content into deleted temp_file
        temp_file = NamedTemporaryFile(delete=True)
        temp_file.write(urlopen(force_bytes(self.guid)).read())
        temp_file.flush()
        # create DjangoFile object
        django_file = DjangoFile(temp_file, name=self.guid.split('/')[-1])
        # choose folder
        if self.parent:
            folder = self.parent.get_or_create_folder(options)
        else:
            folder = options.file_folder
        # import file
        self.file = FileImporter().import_file(file_obj=django_file, folder=folder)
        # set date and owner
        self.file.created_at = self.pub_date
        self.file.owner = self.created_by.user
        self.file.save()
        # return imported file
        self.save()
        return self.file

    def get_or_create_folder(self, options):
        assert self.children.count() > 0
        if self.folder:
            return self.folder
        # do not create sub-folders for slides
        if self.post_type == 'slide':
            self.folder = options.slide_folder
            self.save()
            return self.folder
        parent = options.get_folder(self.post_type)
        self.folder, new = Folder.objects.get_or_create(parent=parent, name=self.title)
        if new:
            self.folder.created_at = self.post_date
            self.folder.owner = self.created_by.user
            self.folder.save()
        self.save()
        return self.folder


@python_2_unicode_compatible
class Options(models.Model):
    name = models.CharField(_('name'), max_length=255, unique=True)

    # global options
    language = models.CharField(_('language'), max_length=15, help_text=_('The language of the content fields.'))

    # article specific options
    article_tree = models.ForeignKey(
        Page, verbose_name=_('tree'), related_name='+',
        help_text=_('All posts will be imported as articles in this tree.'),
        limit_choices_to={
            'publisher_is_draft': False,
            'application_urls': 'CMSArticlesApp',
            'site_id': settings.SITE_ID,
        })
    article_template = models.CharField(
        _('template'), max_length=100,
        choices=settings.CMS_ARTICLES_TEMPLATES,
        default=settings.CMS_ARTICLES_TEMPLATES[0][0])
    article_slot = models.CharField(
        _('slot'), max_length=255, default=settings.CMS_ARTICLES_SLOT,
        help_text=_('The name of placeholder used to create content plugins in.'))
    article_folder = FilerFolderField(
        verbose_name=_('attachments folder'), related_name='+', on_delete=models.SET_NULL, blank=True, null=True,
        help_text=_('Select folder for articles. Subfolder will be created for each article with attachments.'))
    article_redirects = models.BooleanField(
        _('create redirects'), default=True,
        help_text=_('Create django redirects for each article from the old path to the new imported path'))
    article_publish = models.BooleanField(_('publish'), default=False, help_text=_('Publish imported articles.'))

    # page specific options
    page_root = PageField(
        verbose_name=_('root'), related_name='+', on_delete=models.SET_NULL, blank=True, null=True,
        help_text=_('All pages will be imported as sub-pages of this page.'))
    page_template = models.CharField(
        _('template'), max_length=100,
        choices=Page.template_choices, default=Page.TEMPLATE_DEFAULT)
    page_slot = models.CharField(
        _('slot'), max_length=255, default='content',
        help_text=_('The name of placeholder used to create content plugins in.'))
    page_folder = FilerFolderField(
        verbose_name=_('attachments folder'), related_name='+', on_delete=models.SET_NULL, blank=True, null=True,
        help_text=_('Select folder for pages. Subfolder will be created for each page with attachments.'))
    page_redirects = models.BooleanField(
        _('create redirects'), default=True,
        help_text=_('Create django redirects for each page from the old path to the new imported path'))
    page_publish = models.BooleanField(_('publish'), default=False, help_text=_('Publish imported pages.'))

    # file specific options
    gallery_folder = FilerFolderField(
        verbose_name=_('folder'), related_name='+', on_delete=models.SET_NULL, blank=True, null=True,
        help_text=_('Select folder for galleries. Subfolder will be created for each gallery.'))

    # file specific options
    slide_folder = FilerFolderField(
        verbose_name=_('folder'), related_name='+', on_delete=models.SET_NULL, blank=True, null=True,
        help_text=_('Select folder for slides.'))

    # file specific options
    file_folder = FilerFolderField(
        verbose_name=_('folder'), related_name='+', on_delete=models.SET_NULL, blank=True, null=True,
        help_text=_('Select folder for other attachments.'),
    )

    def __str__(self):
        return '{}'.format(self.name)

    class Meta:
        verbose_name = _('options')
        verbose_name_plural = _('options')

    @cached_property
    def folders(self):
        return {
            'post': self.article_folder,
            'page': self.page_folder,
            'gallery': self.gallery_folder,
            'slide': self.slide_folder,
        }

    def get_folder(self, post_type):
        return self.folders.get(post_type, self.file_folder)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2017-03-05 07:53
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cms_articles', '0008_cms_3_4'),
    ]

    operations = [
        migrations.AlterField(
            model_name='articlescategoryplugin',
            name='number',
            field=models.PositiveSmallIntegerField(default=3, validators=[django.core.validators.MinValueValidator(1)], verbose_name='Number of last articles'),
        ),
        migrations.AlterField(
            model_name='articlesplugin',
            name='number',
            field=models.PositiveSmallIntegerField(default=3, validators=[django.core.validators.MinValueValidator(1)], verbose_name='Number of last articles'),
        ),
    ]



from django.db.models import Q


def is_valid_article_slug(article, language, slug):
    """Validates given slug depending on settings.
    """
    from ..models import Title

    qs = Title.objects.filter(slug=slug, language=language)

    if article.pk:
        qs = qs.exclude(Q(language=language) & Q(article=article))
        qs = qs.exclude(article__publisher_public=article)

    if qs.count():
        return False

    return True



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-11 18:21
from __future__ import unicode_literals

import cms.models.fields
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone

from cms_articles.conf import settings


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0013_urlconfrevision'),
    ]

    operations = [
        migrations.CreateModel(
            name='Article',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_by', models.CharField(editable=False, max_length=255, verbose_name='created by')),
                ('changed_by', models.CharField(editable=False, max_length=255, verbose_name='changed by')),
                ('creation_date', models.DateTimeField(auto_now_add=True)),
                ('changed_date', models.DateTimeField(auto_now=True)),
                ('publication_date', models.DateTimeField(blank=True, db_index=True, help_text='When the article should go live. Status must be "Published" for article to go live.', null=True, verbose_name='publication date')),
                ('publication_end_date', models.DateTimeField(blank=True, db_index=True, help_text='When to expire the article. Leave empty to never expire.', null=True, verbose_name='publication end date')),
                ('template', models.CharField(choices=settings.CMS_ARTICLES_TEMPLATES, default=settings.CMS_ARTICLES_TEMPLATES[0][0], help_text='The template used to render the content.', max_length=100, verbose_name='template')),
                ('login_required', models.BooleanField(default=False, verbose_name='login required')),
                ('publisher_is_draft', models.BooleanField(db_index=True, default=True, editable=False)),
                ('languages', models.CharField(blank=True, editable=False, max_length=255, null=True)),
                ('category', models.ForeignKey(help_text='The page the article is accessible at.', on_delete=django.db.models.deletion.CASCADE, related_name='cms_articles', to='cms.Page', verbose_name='category')),
                ('placeholders', models.ManyToManyField(editable=False, related_name='cms_articles', to='cms.Placeholder')),
                ('publisher_public', models.OneToOneField(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='publisher_draft', to='cms_articles.Article')),
            ],
            options={
                'ordering': ('-creation_date',),
                'verbose_name': 'article',
                'verbose_name_plural': 'articles',
                'permissions': (('publish_article', 'Can publish page'),),
            },
        ),
        migrations.CreateModel(
            name='ArticlePlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('template', models.CharField(choices=settings.CMS_ARTICLES_PLUGIN_ARTICLE_TEMPLATES, default=settings.CMS_ARTICLES_PLUGIN_ARTICLE_TEMPLATES[0][0], help_text='The template used to render plugin.', max_length=100, verbose_name='Template')),
                ('article', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='cms_articles.Article', verbose_name='article')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='ArticlesPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('number', models.IntegerField(default=3, verbose_name='Number of last articles')),
                ('template', models.CharField(choices=settings.CMS_ARTICLES_PLUGIN_ARTICLES_TEMPLATES, default=settings.CMS_ARTICLES_PLUGIN_ARTICLES_TEMPLATES[0][0], help_text='The template used to render plugin.', max_length=100, verbose_name='Template')),
                ('category', models.ForeignKey(blank=True, help_text='Keep empty to show articles from current page, if current page is a category.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='cms.Page', verbose_name='category')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='Title',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language', models.CharField(db_index=True, max_length=15, verbose_name='language')),
                ('title', models.CharField(max_length=255, verbose_name='title')),
                ('page_title', models.CharField(blank=True, help_text='overwrite the title (html title tag)', max_length=255, null=True, verbose_name='title')),
                ('menu_title', models.CharField(blank=True, help_text='overwrite the title in the menu', max_length=255, null=True, verbose_name='title')),
                ('meta_description', models.TextField(blank=True, help_text='The text displayed in search engines.', max_length=155, null=True, verbose_name='description')),
                ('slug', models.SlugField(max_length=255, verbose_name='slug')),
                ('creation_date', models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='creation date')),
                ('published', models.BooleanField(default=False, verbose_name='is published')),
                ('publisher_is_draft', models.BooleanField(db_index=True, default=True, editable=False)),
                ('publisher_state', models.SmallIntegerField(db_index=True, default=0, editable=False)),
                ('article', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='title_set', to='cms_articles.Article', verbose_name='article')),
                ('excerpt', cms.models.fields.PlaceholderField(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, slotname='excerpt', to='cms.Placeholder')),
                ('publisher_public', models.OneToOneField(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='publisher_draft', to='cms_articles.Title')),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='title',
            unique_together=set([('language', 'article')]),
        ),
    ]



from cms.cms_toolbars import ADMIN_MENU_IDENTIFIER
from cms.toolbar.items import SubMenu
from cms.toolbar_base import CMSToolbar
from cms.toolbar_pool import toolbar_pool
from cms.utils.i18n import force_language
from cms.utils.urlutils import add_url_parameters, admin_reverse
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _


@toolbar_pool.register
class CMSArticlesToolbar(CMSToolbar):

    def populate(self):
        # Articles item in main menu
        admin_menu = self.toolbar.get_or_create_menu(ADMIN_MENU_IDENTIFIER)
        position = admin_menu.get_alphabetical_insert_position(_('Articles'), SubMenu)
        url = reverse('admin:cms_articles_article_changelist')
        admin_menu.add_sideframe_item(_('Articles'), url=url, position=position)

        # Article menu
        article_menu = self.toolbar.get_or_create_menu('cms-articles', _('Article'))
        self.article = getattr(self.request, 'current_article', None)
        if self.article:
            if self.toolbar.edit_mode:
                url = '{}?language={}'.format(
                    admin_reverse('cms_articles_article_change', args=(self.article.pk,)), self.current_lang)
                article_menu.add_modal_item(_('Article Settings'), url=url)
            else:
                article_menu.add_link_item(_('Edit this article'), url='?edit')
        url = '{}?language={}'.format(admin_reverse('cms_articles_article_add'), self.current_lang)
        if self.request.current_page and self.request.current_page.application_urls == 'CMSArticlesApp':
            url += '&tree={}'.format(self.request.current_page.get_public_object().id)
        article_menu.add_modal_item(_('New Article'), url=url)

    def post_template_populate(self):
        if (self.toolbar.edit_mode and self.article and self.article.has_publish_permission(self.request) and
                self.article.is_dirty(self.current_lang)):
            classes = ['cms-btn-action', 'cms-btn-publish', 'cms-btn-publish-active', 'cms-publish-article']

            title = _('Publish article now')

            params = {}
            params['redirect'] = self.request.path_info

            with force_language(self.current_lang):
                url = admin_reverse('cms_articles_article_publish_article', args=(self.article.pk, self.current_lang))

            url = add_url_parameters(url, params)

            self.toolbar.add_button(title, url=url, extra_classes=classes, side=self.toolbar.RIGHT, disabled=False)

    def request_hook(self):
        pass



import logging
from json import dumps
from xml.etree.ElementTree import ElementTree

from cms.utils.compat.dj import is_installed
from django.utils.timezone import make_aware

from dateutil.parser import parse

from ..conf import settings

if is_installed('django.contrib.redirects'):
    from django.contrib.redirects.models import Redirect
    try:
        from urllib.parse import urlparse
    except ImportError:
        from urlparse import urlparse

    def create_redirect(old_url, new_url):
        old_path = urlparse(old_url).path
        new_path = urlparse(new_url).path
        if old_path != '/' and new_path != old_path:
            redirect = Redirect.objects.get_or_create(
                site_id=settings.SITE_ID,
                old_path=urlparse(old_path).path,
            )[0]
            redirect.new_path = new_path
            redirect.save()
            return redirect
else:
    def create_redirect(old_url, new_url):
        pass


def import_wordpress(xmlfile):
    from .models import Author, Category, Item
    try:
        rss = ElementTree(file=xmlfile).getroot()
        assert rss.tag == 'rss'
    except Exception as e:
        raise Exception('Failed to parse file {}: {}'.format(xmlfile, e))

    imported_items = 0
    errors = []

    # import authors
    authors = {}
    for author in rss.findall('*/{http://wordpress.org/export/1.2/}author'):
        author_id = 'unknown'
        try:
            # first of all try to parse author_id for use in potential error messages
            author_id = int(author.find('{http://wordpress.org/export/1.2/}author_id').text)
            login = author.find('{http://wordpress.org/export/1.2/}author_login').text
            email = author.find('{http://wordpress.org/export/1.2/}author_email').text
            first_name = author.find('{http://wordpress.org/export/1.2/}author_first_name').text
            last_name = author.find('{http://wordpress.org/export/1.2/}author_last_name').text
        except Exception as e:
            error = 'Failed to parse author with author_id {}: {}'.format(author_id, e)
            logging.warning(error)
            errors.append(error)
            continue
        try:
            author = Author.objects.get_or_create(
                author_id=author_id,
                login=login,
                email=email,
                first_name=first_name,
                last_name=last_name,
            )[0]
        except Exception as e:
            error = 'Failed to save author with author_id {}: {}'.format(author_id, e)
            logging.warning(error)
            errors.append(error)
            continue
        authors[login] = author

    # import categories
    categories = {}
    for category in rss.findall('*/{http://wordpress.org/export/1.2/}category'):
        term_id = 'unknown'
        try:
            # first of all try to parse term_id for use in potential error messages
            term_id = int(category.find('{http://wordpress.org/export/1.2/}term_id').text)
            name = category.find('{http://wordpress.org/export/1.2/}cat_name').text
            slug = category.find('{http://wordpress.org/export/1.2/}category_nicename').text
            parent = category.find('{http://wordpress.org/export/1.2/}category_parent').text
        except Exception as e:
            error = 'Failed to parse category with term_id {}: {}'.format(term_id, e)
            logging.warning(error)
            errors.append(error)
            raise
            continue
        try:
            category = Category.objects.get_or_create(
                term_id=term_id,
                name=name,
                slug=slug,
                parent=parent,
            )[0]
        except Exception as e:
            error = 'Failed to save category with term_id {}: {}'.format(term_id, e)
            logging.warning(error)
            errors.append(error)
            continue
        categories[slug] = category

    # import items
    for item in rss.findall('*/item'):
        post_id = 'unknown'
        try:
            # first of all try to parse post_id for use in potential error messages
            post_id = int(item.find('{http://wordpress.org/export/1.2/}post_id').text)
            title = item.find('title').text or ''
            link = item.find('link').text or ''
            pub_date = parse(item.find('pubDate').text)
            created_by = item.find('{http://purl.org/dc/elements/1.1/}creator').text
            guid = item.find('guid').text
            description = item.find('description').text or ''
            content = item.find('{http://purl.org/rss/1.0/modules/content/}encoded').text or ''
            excerpt = item.find('{http://wordpress.org/export/1.2/excerpt/}encoded').text or ''
            post_date = make_aware(parse(item.find('{http://wordpress.org/export/1.2/}post_date').text))
            post_name = item.find('{http://wordpress.org/export/1.2/}post_name').text or ''
            status = item.find('{http://wordpress.org/export/1.2/}status').text
            post_parent = int(item.find('{http://wordpress.org/export/1.2/}post_parent').text)
            post_type = item.find('{http://wordpress.org/export/1.2/}post_type').text
            postmeta = dumps(dict(
                (
                    pm.find('{http://wordpress.org/export/1.2/}meta_key').text,
                    pm.find('{http://wordpress.org/export/1.2/}meta_value').text,
                )
                for pm in item.findall('{http://wordpress.org/export/1.2/}postmeta')
            ))
            cats = [
                categories[cat.attrib['nicename']]
                for cat in item.findall('category')
                if cat.attrib['nicename'] in categories
            ]
        except Exception as e:
            error = 'Failed to parse item with post_id {}: {}'.format(post_id, e)
            logging.warning(error)
            errors.append(error)
            continue
        try:
            item = Item.objects.create(
                title=title,
                link=link,
                pub_date=pub_date,
                created_by=authors[created_by],
                guid=guid,
                description=description,
                content=content,
                excerpt=excerpt,
                post_id=post_id,
                post_date=post_date,
                post_name=post_name,
                status=status,
                post_parent=post_parent,
                post_type=post_type,
                postmeta=postmeta,
            )
            item.categories = cats
        except Exception as e:
            error = 'Failed to save item with post_id {}: {}'.format(post_id, e)
            logging.warning(error)
            errors.append(error)
            continue
        imported_items += 1
    return {
        'authors': len(authors),
        'categories': len(categories),
        'items': imported_items,
        'errors': errors,
    }



# -*- coding: utf-8 -*-
from cms import constants
from cms.exceptions import (
    LanguageError, PublicIsUnmodifiable, PublicVersionNeeded,
)
from cms.models import Page
from cms.utils import i18n
from cms.utils.copy_plugins import copy_plugins_to
from cms.utils.helpers import reversion_register
from django.db import models
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.html import strip_tags
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import get_language

from ..conf import settings
from .attribute import Attribute
from .category import Category
from .managers import ArticleManager


@python_2_unicode_compatible
class Article(models.Model):
    tree = models.ForeignKey(
        Page, verbose_name=_('tree'), related_name='cms_articles',
        help_text=_('The page the article is accessible at.'), limit_choices_to={
            'publisher_is_draft': False,
            'application_urls': 'CMSArticlesApp',
            'site_id': settings.SITE_ID,
        })
    template = models.CharField(
        _('template'), max_length=100,
        choices=settings.CMS_ARTICLES_TEMPLATES, default=settings.CMS_ARTICLES_TEMPLATES[0][0],
        help_text=_('The template used to render the content.'))
    attributes = models.ManyToManyField(Attribute, verbose_name=_('attributes'), related_name='articles', blank=True)
    categories = models.ManyToManyField(Category, verbose_name=_('categories'), related_name='articles', blank=True)
    created_by = models.CharField(_('created by'), max_length=constants.PAGE_USERNAME_MAX_LENGTH, editable=False)
    changed_by = models.CharField(_('changed by'), max_length=constants.PAGE_USERNAME_MAX_LENGTH, editable=False)
    creation_date = models.DateTimeField(auto_now_add=True)
    changed_date = models.DateTimeField(auto_now=True)
    publication_date = models.DateTimeField(
        _('publication date'), null=True, blank=True, db_index=True,
        help_text=_('When the article should go live. Status must be "Published" for article to go live.'))
    publication_end_date = models.DateTimeField(
        _('publication end date'), null=True, blank=True, db_index=True,
        help_text=_('When to expire the article. Leave empty to never expire.'))
    order_date = models.DateTimeField(_('publication or creation time'), auto_now_add=True, editable=False)
    login_required = models.BooleanField(_('login required'), default=False)

    # Placeholders (plugins)
    placeholders = models.ManyToManyField('cms.Placeholder', related_name='cms_articles', editable=False)

    # Publisher fields
    publisher_is_draft = models.BooleanField(default=True, editable=False, db_index=True)
    # This is misnamed - the one-to-one relation is populated on both ends
    publisher_public = models.OneToOneField('self', related_name='publisher_draft', null=True, editable=False)
    languages = models.CharField(max_length=255, editable=False, blank=True, null=True)

    # If the draft is loaded from a reversion version save the revision id here.
    revision_id = models.PositiveIntegerField(default=0, editable=False)

    # Managers
    objects = ArticleManager()

    class Meta:
        permissions = (('publish_article', 'Can publish article'),)
        app_label = 'cms_articles'
        verbose_name = _('article')
        verbose_name_plural = _('articles')
        ordering = ('-order_date',)

    def __str__(self):
        try:
            title = self.get_title(fallback=True)
        except LanguageError:
            try:
                title = self.title_set.all()[0]
            except IndexError:
                title = None
        if title is None:
            title = u''
        return force_text(title)

    def __repr__(self):
        # This is needed to solve the infinite recursion when
        # adding new articles.
        return object.__repr__(self)

    def is_dirty(self, language):
        state = self.get_publisher_state(language)
        return state == constants.PUBLISHER_STATE_DIRTY

    def get_absolute_url(self, language=None, fallback=True):
        if not language:
            language = get_language()
        return '{}{}{}{}'.format(
            self.tree.get_absolute_url(language, fallback),
            '' if settings.APPEND_SLASH else '/',
            self.get_slug(language, fallback),
            '/' if settings.APPEND_SLASH else '',
        )

    def get_public_url(self, language=None, fallback=True):
        '''
        Returns the URL of the published version of the current article.
        Returns empty string if the article is not published.
        '''
        try:
            return self.get_public_object().get_absolute_url(language, fallback)
        except:
            return ''

    def get_draft_url(self, language=None, fallback=True):
        '''
        Returns the URL of the draft version of the current article.
        Returns empty string if the draft article is not available.
        '''
        try:
            return self.get_draft_object().get_absolute_url(language, fallback)
        except:
            return ''

    def revert_to_live(self, language):
        """Revert the draft version to the same state as the public version
        """
        if not self.publisher_is_draft:
            # Revert can only be called on draft articles
            raise PublicIsUnmodifiable('The public instance cannot be reverted. Use draft.')

        if not self.publisher_public:
            raise PublicVersionNeeded('A public version of this article is needed')

        public = self.publisher_public
        public._copy_titles(self, language)
        public._copy_contents(self, language)
        public._copy_attributes(self)

        self.title_set.filter(language=language).update(
            publisher_state=constants.PUBLISHER_STATE_DEFAULT,
            published=True,
        )

        self._publisher_keep_state = True
        self.save()

    def _copy_titles(self, target, language):
        '''
        Copy all the titles to a new article (which must have a pk).
        :param target: The article where the new titles should be stored
        '''
        from .title import Title

        old_titles = dict(target.title_set.filter(language=language).values_list('language', 'pk'))
        for title in self.title_set.filter(language=language):
            old_pk = title.pk
            # If an old title exists, overwrite. Otherwise create new
            title.pk = old_titles.pop(title.language, None)
            title.article = target
            title.publisher_is_draft = target.publisher_is_draft
            title.publisher_public_id = old_pk
            title.publisher_state = constants.PUBLISHER_STATE_DEFAULT
            title.published = True
            title._publisher_keep_state = True
            title.save()

            old_title = Title.objects.get(pk=old_pk)
            old_title.publisher_public = title
            old_title.publisher_state = title.publisher_state
            old_title.published = True
            old_title._publisher_keep_state = True
            old_title.save()
            if hasattr(self, 'title_cache'):
                self.title_cache[language] = old_title
        if old_titles:
            Title.objects.filter(id__in=old_titles.values()).delete()

    def _copy_contents(self, target, language):
        '''
        Copy all the plugins to a new article.
        :param target: The article where the new content should be stored
        '''
        # TODO: Make this into a 'graceful' copy instead of deleting and overwriting
        # copy the placeholders (and plugins on those placeholders!)
        from cms.models.pluginmodel import CMSPlugin
        from cms.plugin_pool import plugin_pool

        plugin_pool.set_plugin_meta()
        for plugin in CMSPlugin.objects.filter(placeholder__cms_articles=target, language=language).order_by('-depth'):
            inst, cls = plugin.get_plugin_instance()
            if inst and getattr(inst, 'cmsplugin_ptr_id', False):
                inst.cmsplugin_ptr = plugin
                inst.cmsplugin_ptr._no_reorder = True
                inst.delete(no_mp=True)
            else:
                plugin._no_reorder = True
                plugin.delete(no_mp=True)
        new_phs = []
        target_phs = target.placeholders.all()
        for ph in self.get_placeholders():
            plugins = ph.get_plugins_list(language)
            found = False
            for target_ph in target_phs:
                if target_ph.slot == ph.slot:
                    ph = target_ph
                    found = True
                    break
            if not found:
                ph.pk = None  # make a new instance
                ph.save()
                new_phs.append(ph)
                # update the article copy
            if plugins:
                copy_plugins_to(plugins, ph)
        target.placeholders.add(*new_phs)

    def _copy_attributes(self, target):
        target.tree = self.tree
        target.template = self.template
        target.publication_date = self.publication_date
        target.publication_end_date = self.publication_end_date
        target.login_required = self.login_required

    def _copy_relations(self, target):
        target.attributes = self.attributes.all()
        target.categories = self.categories.all()

    def delete(self, *args, **kwargs):
        articles = [self.pk]
        if self.publisher_public_id:
            articles.append(self.publisher_public_id)
        self.__class__.objects.filter(pk__in=articles).delete()

    def save(self, no_signals=False, commit=True, **kwargs):
        '''
        Args:
            commit: True if model should be really saved
        '''
        # delete template cache
        if hasattr(self, '_template_cache'):
            delattr(self, '_template_cache')

        created = not bool(self.pk)

        from cms.utils.permissions import get_current_user

        user = get_current_user()

        if user:
            try:
                changed_by = force_text(user)
            except AttributeError:
                # AnonymousUser may not have USERNAME_FIELD
                changed_by = 'anonymous'
            else:
                # limit changed_by and created_by to avoid problems with Custom User Model
                if len(changed_by) > constants.PAGE_USERNAME_MAX_LENGTH:
                    changed_by = u'{0}... (id={1})'.format(
                        changed_by[:constants.PAGE_USERNAME_MAX_LENGTH - 15],
                        user.pk,
                    )

            self.changed_by = changed_by

        else:
            self.changed_by = 'script'
        if created:
            self.created_by = self.changed_by

        self.order_date = self.publication_date or self.creation_date

        if commit:
            super(Article, self).save(**kwargs)

    def save_base(self, *args, **kwargs):
        '''Overridden save_base. If an instance is draft, and was changed, mark
        it as dirty.

        Dirty flag is used for changed nodes identification when publish method
        takes place. After current changes are published, state is set back to
        PUBLISHER_STATE_DEFAULT (in publish method).
        '''
        keep_state = getattr(self, '_publisher_keep_state', None)
        if self.publisher_is_draft and not keep_state and self.is_new_dirty():
            self.title_set.all().update(publisher_state=constants.PUBLISHER_STATE_DIRTY)
        if keep_state:
            delattr(self, '_publisher_keep_state')
        return super(Article, self).save_base(*args, **kwargs)

    def is_new_dirty(self):
        if self.pk:
            fields = [
                'publication_date', 'publication_end_date', 'tree', 'template', 'login_required',
            ]
            try:
                old_article = Article.objects.get(pk=self.pk)
            except Article.DoesNotExist:
                return True
            for field in fields:
                old_val = getattr(old_article, field)
                new_val = getattr(self, field)
                if not old_val == new_val:
                    return True
            if not self.attributes.all() == old_article.attributes.all():
                return True
            if not self.categories.all() == old_article.categories.all():
                return True
            return False
        return True

    def is_published(self, language, force_reload=False):
        return self.get_title_obj(language, False, force_reload=force_reload).published

    def get_publisher_state(self, language, force_reload=False):
        try:
            return self.get_title_obj(language, False, force_reload=force_reload).publisher_state
        except AttributeError:
            return None

    def set_publisher_state(self, language, state, published=None):
        title = self.title_set.get(language=language)
        title.publisher_state = state
        if published is not None:
            title.published = published
        title._publisher_keep_state = True
        title.save()
        if hasattr(self, 'title_cache') and language in self.title_cache:
            self.title_cache[language].publisher_state = state
        return title

    def publish(self, language):
        # Publish can only be called on draft articles
        if not self.publisher_is_draft:
            raise PublicIsUnmodifiable('The public instance cannot be published. Use draft.')

        if not self.pk:
            self.save()

        if self.publisher_public_id:
            public_article = self.publisher_public
        else:
            public_article = Article(created_by=self.created_by)

        if not self.publication_date:
            self.publication_date = now()

        self._copy_attributes(public_article)

        # we need to set relate this new public copy to its draft article (self)
        public_article.publisher_public = self
        public_article.publisher_is_draft = False
        public_article.save()

        public_article = public_article.reload()

        # The target article now has a pk, so can be used as a target
        self._copy_titles(public_article, language)
        self._copy_contents(public_article, language)
        self._copy_relations(public_article)

        self.publisher_public = public_article
        self._publisher_keep_state = True
        self.save()

        from ..signals import post_publish_article
        post_publish_article.send(sender=Article, instance=self, language=language)

        return True

    def unpublish(self, language):
        '''
        Removes this article from the public site
        :returns: True if this article was successfully unpublished
        '''
        # Publish can only be called on draft articles
        if not self.publisher_is_draft:
            raise PublicIsUnmodifiable('The public instance cannot be unpublished. Use draft.')

        # First, make sure we are in the correct state
        title = self.title_set.get(language=language)
        public_title = title.publisher_public
        title.published = False
        title.publisher_state = constants.PUBLISHER_STATE_DIRTY
        title.save()
        if hasattr(self, 'title_cache'):
            self.title_cache[language] = title
        public_title.published = False

        public_title.save()
        public_article = self.publisher_public
        public_placeholders = public_article.get_placeholders()
        for pl in public_placeholders:
            pl.cmsplugin_set.filter(language=language).delete()
        public_article.save()
        # trigger update home
        self.save()

        from cms.signals import post_unpublish
        post_unpublish.send(sender=Article, instance=self, language=language)

        return True

    def get_draft_object(self):
        if not self.publisher_is_draft:
            return self.publisher_draft
        return self

    def get_public_object(self):
        if not self.publisher_is_draft:
            return self
        return self.publisher_public

    def get_published_object(self):
        if self.publisher_is_draft:
            public_id = self.publisher_public_id
        else:
            public_id = self.id
        return Article.objects.public().published().filter(id=public_id).first()

    def get_languages(self):
        if self.languages:
            return sorted(self.languages.split(','))
        else:
            return []

    def get_published_languages(self):
        if self.publisher_is_draft:
            return self.get_languages()
        return sorted([language for language in self.get_languages() if self.is_published(language)])

    # Title object access

    def get_title_obj(self, language=None, fallback=True, force_reload=False):
        language = self._get_title_cache(language, fallback, force_reload)
        if language in self.title_cache:
            return self.title_cache[language]

        from .title import Title
        title = Title()
        title.article = self
        title.language = language
        return title

    def get_title_obj_attribute(self, attrname, language=None, fallback=True, force_reload=False):
        '''Helper function for getting attribute or None from wanted/current title.
        '''
        try:
            attribute = getattr(self.get_title_obj(
                language, fallback, force_reload), attrname)
            return attribute
        except AttributeError:
            return None

    def get_slug(self, language=None, fallback=True, force_reload=False):
        '''
        get the slug of the article depending on the given language
        '''
        return self.get_title_obj_attribute('slug', language, fallback, force_reload)

    def get_title(self, language=None, fallback=True, force_reload=False):
        '''
        get the title of the article depending on the given language
        '''
        return self.get_title_obj_attribute('title', language, fallback, force_reload)

    def get_image(self, language=None, fallback=True, force_reload=False):
        '''
        get the image of the article depending on the given language
        '''
        return self.get_title_obj_attribute('image', language, fallback, force_reload)

    def get_description(self, language=None, fallback=True, force_reload=False):
        '''
        get description of the article depending on the given language
        '''
        return self.get_title_obj_attribute('description', language, fallback, force_reload)

    def get_placeholders(self):
        if not hasattr(self, '_placeholder_cache'):
            self._placeholder_cache = self.placeholders.all()
        return self._placeholder_cache

    def get_changed_date(self, language=None, fallback=True, force_reload=False):
        '''
        get when this article was last updated
        '''
        return self.changed_date

    def get_changed_by(self, language=None, fallback=True, force_reload=False):
        '''
        get user who last changed this article
        '''
        return self.changed_by

    def get_page_title(self, language=None, fallback=True, force_reload=False):
        '''
        get the page title of the article depending on the given language
        '''
        page_title = self.get_title_obj_attribute('page_title', language, fallback, force_reload)
        if not page_title:
            return self.get_title(language, True, force_reload)
        return page_title

    def get_menu_title(self, language=None, fallback=True, force_reload=False):
        '''
        get the menu title of the article depending on the given language
        '''
        menu_title = self.get_title_obj_attribute('menu_title', language, fallback, force_reload)
        if not menu_title:
            return self.get_title(language, True, force_reload)
        return menu_title

    def get_meta_description(self, language=None, fallback=True, force_reload=False):
        '''
        get content for the description meta tag for the article depending on the given language
        '''
        return (
            self.get_title_obj_attribute('meta_description', language, fallback, force_reload) or
            strip_tags(self.get_title_obj_attribute('description', language, fallback, force_reload)))

    def _get_title_cache(self, language, fallback, force_reload):
        if not language:
            language = get_language()
        load = False
        if not hasattr(self, 'title_cache') or force_reload:
            load = True
            self.title_cache = {}
        elif language not in self.title_cache:
            if fallback:
                fallback_langs = i18n.get_fallback_languages(language)
                for lang in fallback_langs:
                    if lang in self.title_cache:
                        return lang
            load = True
        if load:
            from .title import Title

            titles = Title.objects.filter(article=self)
            for title in titles:
                self.title_cache[title.language] = title
            if language in self.title_cache:
                return language
            else:
                if fallback:
                    fallback_langs = i18n.get_fallback_languages(language)
                    for lang in fallback_langs:
                        if lang in self.title_cache:
                            return lang
        return language

    def get_template(self):
        return self.template

    def has_change_permission(self, request, user=None):
        if not user:
            user = request.user
        return user.has_perm('cms_articles.change_article')

    def has_delete_permission(self, request, user=None):
        if not user:
            user = request.user
        return user.has_perm('cms_articles.delete_article')

    def has_publish_permission(self, request, user=None):
        if not user:
            user = request.user
        return user.has_perm('cms_articles.publish_article')

    def has_add_permission(self, request, user=None):
        if not user:
            user = request.user
        return user.has_perm('cms_articles.add_article')

    def reload(self):
        '''
        Reload a article from the database
        '''
        return Article.objects.get(pk=self.pk)

    def rescan_placeholders(self):
        '''
        Rescan and if necessary create placeholders in the current template.
        '''
        # inline import to prevent circular imports
        from cms.models.placeholdermodel import Placeholder
        from ..utils.placeholder import get_placeholders

        placeholders = get_placeholders(self.template)
        found = {}
        for placeholder in self.placeholders.all():
            if placeholder.slot in placeholders:
                found[placeholder.slot] = placeholder
        for placeholder_name in placeholders:
            if placeholder_name not in found:
                placeholder = Placeholder.objects.create(slot=placeholder_name)
                self.placeholders.add(placeholder)
                found[placeholder_name] = placeholder
        return found

    def get_xframe_options(self):
        return self.tree.get_xframe_options()


def _reversion():
    exclude_fields = [
        'publisher_is_draft',
        'publisher_public',
        'publisher_state',
    ]

    reversion_register(
        Page,
        follow=["title_set", "placeholders"],
        exclude_fields=exclude_fields
    )


_reversion()



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-20 12:28
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cms_articles', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='title',
            name='excerpt',
        ),
    ]



# -*- coding: utf-8 -*-
from cms.exceptions import PlaceholderNotFound


class ArticlesContentRenderer(object):

    def __init__(self, content_renderer):
        self.content_renderer = content_renderer
        self._placeholders_by_article_cache = {}

    def render_article_placeholder(self, slot, context, nodelist=None):
        current_article = self.content_renderer.request.current_article

        if not current_article:
            return ''

        content = self._render_article_placeholder(
            context=context,
            slot=slot,
            article=current_article,
            editable=True,
            nodelist=nodelist,
        )

        return content

    def _get_article_placeholder(self, context, article, slot):
        """
        Returns a Placeholder instance attached to article that
        matches the given slot.

        A PlaceholderNotFound is raised if the placeholder is
        not present on the article template.
        """
        placeholder_cache = self._placeholders_by_article_cache

        if article.pk not in placeholder_cache:
            # Instead of loading plugins for this one placeholder
            # try and load them for all placeholders on the article.
            self._preload_placeholders_for_article(article)

        try:
            placeholder = placeholder_cache[article.pk][slot]
        except KeyError:
            message = '"%s" placeholder not found' % slot
            raise PlaceholderNotFound(message)
        return placeholder

    def _render_article_placeholder(self, context, slot, article, editable=True, nodelist=None):
        """
        Renders a placeholder attached to a article.
        """
        try:
            placeholder = self._get_article_placeholder(context, article, slot)
        except PlaceholderNotFound:
            if nodelist:
                return nodelist.render(context)
            return ''

        content = self.content_renderer.render_placeholder(
            placeholder,
            context=context,
            editable=editable,
            use_cache=True,
            nodelist=nodelist,
        )
        return content

    def _preload_placeholders_for_article(self, article):
        """
        Populates the internal plugin cache of each placeholder
        in the given article if the placeholder has not been
        previously cached.
        """
        from cms.utils.plugins import assign_plugins

        site_id = article.tree.site_id
        placeholders = article.rescan_placeholders().values()

        if self.content_renderer.placeholder_cache_is_enabled():
            _cached_content = self.content_renderer._get_cached_placeholder_content
            # Only prefetch placeholder plugins if the placeholder
            # has not been cached.
            placeholders_to_fetch = [
                placeholder for placeholder in placeholders
                if _cached_content(placeholder, site_id, self.content_renderer.request_language) is None]
        else:
            # cache is disabled, prefetch plugins for all
            # placeholders in the article.
            placeholders_to_fetch = placeholders

        if placeholders_to_fetch:
            assign_plugins(
                request=self.content_renderer.request,
                placeholders=placeholders_to_fetch,
                template=article.get_template(),
                lang=self.content_renderer.request_language,
            )

        # Internal cache mapping placeholder slots
        # to placeholder instances.
        article_placeholder_cache = {}

        for placeholder in placeholders:
            # Save a query when the placeholder toolbar is rendered.
            placeholder.article = article
            article_placeholder_cache[placeholder.slot] = placeholder

        self._placeholders_by_article_cache[article.pk] = article_placeholder_cache



import copy
import re
import sys
import warnings
from threading import local

from cms.admin.placeholderadmin import PlaceholderAdminMixin
from cms.constants import PUBLISHER_STATE_PENDING
from cms.models import CMSPlugin, StaticPlaceholder
from cms.utils import get_language_from_request
from cms.utils.conf import get_cms_setting
from cms.utils.i18n import (
    force_language, get_language_list, get_language_object, get_language_tuple,
)
from cms.utils.urlutils import admin_reverse
from django.conf.urls import url
from django.contrib import admin, messages
from django.contrib.admin.models import CHANGE, LogEntry
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import PermissionDenied, ValidationError
from django.db import router, transaction
from django.http import Http404, HttpResponseForbidden, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.template.defaultfilters import escape
from django.template.loader import get_template
from django.utils.decorators import method_decorator
from django.utils.encoding import force_text
from django.utils.six.moves.urllib.parse import unquote
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.http import require_POST

from ..api import add_content
from ..conf import settings
from ..models import Article, Title
from .forms import ArticleCreateForm, ArticleForm

try:
    from django.contrib.admin.utils import get_deleted_objects
except ImportError:
    from django.contrib.admin.util import get_deleted_objects


require_POST = method_decorator(require_POST)

_thread_locals = local()


class ArticleAdmin(PlaceholderAdminMixin, admin.ModelAdmin):
    change_list_template = 'admin/cms_articles/article_changelist.html'
    search_fields = ('=id', 'title_set__slug', 'title_set__title', 'title_set__description')
    list_display = ('__str__', 'order_date', 'preview_link') + tuple(
        'lang_{}'.format(lang) for lang in get_language_list()
    )
    list_filter = ['tree', 'attributes', 'categories', 'template', 'changed_by']
    date_hierarchy = 'order_date'
    filter_horizontal = ['attributes', 'categories']

    preview_template = get_template('admin/cms_articles/article/change_list_preview.html')

    def preview_link(self, obj):
        return self.preview_template.render({
            'article': obj,
            'lang': _thread_locals.language,
            'request': _thread_locals.request,
        })
    preview_link.short_description = _('Show')
    preview_link.allow_tags = True

    lang_template = get_template('admin/cms_articles/article/change_list_lang.html')

    def __getattr__(self, name):
        if name.startswith('lang_'):
            lang = name[len('lang_'):]

            def lang_dropdown(obj):
                return self.lang_template.render({
                    'article': obj,
                    'lang': lang,
                    'language': _thread_locals.language,
                    'has_change_permission': obj.has_change_permission(_thread_locals.request),
                    'has_publish_permission': obj.has_publish_permission(_thread_locals.request),
                    'request': _thread_locals.request,
                })

            lang_dropdown.short_description = lang
            lang_dropdown.allow_tags = True
            return lang_dropdown
        raise AttributeError(name)

    def get_fieldsets(self, request, obj=None):
        language_dependent = ['title', 'slug', 'description', 'content', 'page_title',
                              'menu_title', 'meta_description', 'image']
        if obj:
            language_dependent.remove('content')
        return [
            (None, {'fields': ['tree', 'template']}),
            (_('Language dependent settings'), {'fields': language_dependent}),
            (_('Other settings'), {'fields': ['attributes', 'categories', 'publication_date',
                                              'publication_end_date', 'login_required']}),
        ]

    def get_urls(self):
        """Get the admin urls
        """
        info = '%s_%s' % (self.model._meta.app_label, self.model._meta.model_name)

        def pat(regex, fn):
            return url(regex, self.admin_site.admin_view(fn), name='%s_%s' % (info, fn.__name__))

        url_patterns = [
            pat(r'^([0-9]+)/delete-translation/$', self.delete_translation),
            pat(r'^([0-9]+)/([a-z\-]+)/publish/$', self.publish_article),
            pat(r'^([0-9]+)/([a-z\-]+)/unpublish/$', self.unpublish),
            pat(r'^([0-9]+)/([a-z\-]+)/preview/$', self.preview_article),
        ]

        url_patterns += super(ArticleAdmin, self).get_urls()
        return url_patterns

    def get_queryset(self, request):
        return super(ArticleAdmin, self).get_queryset(request).filter(
            tree__site_id=settings.SITE_ID,
            publisher_is_draft=True,
        )

    def save_model(self, request, obj, form, change):
        new = obj.id is None
        super(ArticleAdmin, self).save_model(request, obj, form, change)
        Title.objects.set_or_create(request, obj, form, form.cleaned_data['language'])
        if new and form.cleaned_data['content']:
            add_content(
                obj,
                language=form.cleaned_data['language'],
                slot=settings.CMS_ARTICLES_SLOT,
                content=form.cleaned_data['content'],
            )

    def get_form(self, request, obj=None, **kwargs):
        """
        Get ArticleForm for the Article model and modify its fields depending on
        the request.
        """
        language = get_language_from_request(request)
        form = super(ArticleAdmin, self).get_form(
            request, obj,
            form=(obj and ArticleForm or ArticleCreateForm),
            **kwargs
        )
        # get_form method operates by overriding initial fields value which
        # may persist across invocation. Code below deepcopies fields definition
        # to avoid leaks
        for field in form.base_fields.keys():
            form.base_fields[field] = copy.deepcopy(form.base_fields[field])

        if 'language' in form.base_fields:
            form.base_fields['language'].initial = language

        if obj:
            title_obj = obj.get_title_obj(language=language, fallback=False, force_reload=True)

            if hasattr(title_obj, 'id'):
                for name in ('title', 'description', 'page_title', 'menu_title', 'meta_description', 'image'):
                    if name in form.base_fields:
                        form.base_fields[name].initial = getattr(title_obj, name)
                try:
                    slug = re.search(settings.CMS_ARTICLES_SLUG_REGEXP, title_obj.slug).groups()[0]
                except:
                    warnings.warn('Failed to parse slug from CMS_ARTICLES_SLUG_REGEXP. '
                                  'It probably doesn\'t correspond to CMS_ARTICLES_SLUG_FORMAT.')
                    slug = title_obj.slug
                form.base_fields['slug'].initial = slug

        return form

    def get_unihandecode_context(self, language):
        if language[:2] in get_cms_setting('UNIHANDECODE_DECODERS'):
            uhd_lang = language[:2]
        else:
            uhd_lang = get_cms_setting('UNIHANDECODE_DEFAULT_DECODER')
        uhd_host = get_cms_setting('UNIHANDECODE_HOST')
        uhd_version = get_cms_setting('UNIHANDECODE_VERSION')
        if uhd_lang and uhd_host and uhd_version:
            uhd_urls = [
                '%sunihandecode-%s.core.min.js' % (uhd_host, uhd_version),
                '%sunihandecode-%s.%s.min.js' % (uhd_host, uhd_version, uhd_lang),
            ]
        else:
            uhd_urls = []
        return {'unihandecode_lang': uhd_lang, 'unihandecode_urls': uhd_urls}

    def changelist_view(self, request, extra_context=None):
        _thread_locals.request = request
        _thread_locals.language = get_language_from_request(request)
        return super(ArticleAdmin, self).changelist_view(request, extra_context=extra_context)

    def add_view(self, request, form_url='', extra_context=None):
        extra_context = self.update_language_tab_context(request, context=extra_context)
        extra_context.update(self.get_unihandecode_context(extra_context['language']))
        return super(ArticleAdmin, self).add_view(request, form_url, extra_context=extra_context)

    def change_view(self, request, object_id, form_url='', extra_context=None):
        extra_context = self.update_language_tab_context(request, context=extra_context)
        language = extra_context['language']
        extra_context.update(self.get_unihandecode_context(language))
        response = super(ArticleAdmin, self).change_view(request, object_id,
                                                         form_url=form_url, extra_context=extra_context)
        if language and response.status_code == 302 and response._headers['location'][1] == request.path_info:
            location = response._headers['location']
            response._headers['location'] = (location[0], '%s?language=%s' % (location[1], language))
        return response

    def render_change_form(self, request, context, add=False, change=False, form_url='', obj=None):
        # add context variables
        filled_languages = []
        if obj:
            filled_languages = [t[0] for t in obj.title_set.filter(title__isnull=False).values_list('language')]
        allowed_languages = [lang[0] for lang in get_language_tuple()]
        context.update({
            'filled_languages': [lang for lang in filled_languages if lang in allowed_languages],
        })
        return super(ArticleAdmin, self).render_change_form(request, context, add, change, form_url, obj)

    def update_language_tab_context(self, request, context=None):
        if not context:
            context = {}
        language = get_language_from_request(request)
        languages = get_language_tuple()
        context.update({
            'language': language,
            'languages': languages,
            'language_tabs': languages,
            'show_language_tabs': len(list(languages)) > 1,
        })
        return context

    @require_POST
    @transaction.atomic
    def publish_article(self, request, article_id, language):
        try:
            article = Article.objects.get(id=article_id, publisher_is_draft=True)
        except Article.DoesNotExist:
            article = None

        # ensure user has permissions to publish this article
        if article:
            if not self.has_change_permission(request):
                return HttpResponseForbidden(force_text(_('You do not have permission to publish this article')))
            article.publish(language)
        statics = request.GET.get('statics', '')
        if not statics and not article:
            raise Http404('No article or stack found for publishing.')
        all_published = True
        if statics:
            static_ids = statics.split(',')
            for pk in static_ids:
                static_placeholder = StaticPlaceholder.objects.get(pk=pk)
                published = static_placeholder.publish(request, language)
                if not published:
                    all_published = False
        if article:
            if all_published:
                messages.info(request, _('The content was successfully published.'))
                LogEntry.objects.log_action(
                    user_id=request.user.id,
                    content_type_id=ContentType.objects.get_for_model(Article).pk,
                    object_id=article_id,
                    object_repr=article.get_title(language),
                    action_flag=CHANGE,
                )
            else:
                messages.warning(request, _('There was a problem publishing your content'))

        if 'redirect' in request.GET:
            return HttpResponseRedirect(request.GET['redirect'])

        referrer = request.META.get('HTTP_REFERER', '')
        path = admin_reverse('cms_articles_article_changelist')
        if request.GET.get('redirect_language'):
            path = '%s?language=%s&article_id=%s' % (
                path,
                request.GET.get('redirect_language'),
                request.GET.get('redirect_article_id')
            )
        if admin_reverse('index') not in referrer:
            if all_published:
                if article:
                    if article.get_publisher_state(language) == PUBLISHER_STATE_PENDING:
                        path = article.get_absolute_url(language, fallback=True)
                    else:
                        public_article = Article.objects.get(publisher_public=article.pk)
                        path = '%s?%s' % (
                            public_article.get_absolute_url(language, fallback=True),
                            get_cms_setting('CMS_TOOLBAR_URL__EDIT_OFF')
                        )
                else:
                    path = '%s?%s' % (referrer, get_cms_setting('CMS_TOOLBAR_URL__EDIT_OFF'))
            else:
                path = '/?%s' % get_cms_setting('CMS_TOOLBAR_URL__EDIT_OFF')

        return HttpResponseRedirect(path)

    @require_POST
    @transaction.atomic
    def unpublish(self, request, article_id, language):
        """
        Publish or unpublish a language of a article
        """
        article = get_object_or_404(self.model, pk=article_id)
        if not article.has_publish_permission(request):
            return HttpResponseForbidden(force_text(_('You do not have permission to unpublish this article')))
        if not article.publisher_public_id:
            return HttpResponseForbidden(force_text(_('This article was never published')))
        try:
            article.unpublish(language)
            message = _('The %(language)s article "%(article)s" was successfully unpublished') % {
                'language': get_language_object(language)['name'], 'article': article}
            messages.info(request, message)
            LogEntry.objects.log_action(
                user_id=request.user.id,
                content_type_id=ContentType.objects.get_for_model(Article).pk,
                object_id=article_id,
                object_repr=article.get_title(),
                action_flag=CHANGE,
                change_message=message,
            )
        except RuntimeError:
            exc = sys.exc_info()[1]
            messages.error(request, exc.message)
        except ValidationError:
            exc = sys.exc_info()[1]
            messages.error(request, exc.message)
        path = admin_reverse('cms_articles_article_changelist')
        if request.GET.get('redirect_language'):
            path = '%s?language=%s&article_id=%s' % (
                path,
                request.GET.get('redirect_language'),
                request.GET.get('redirect_article_id')
            )
        return HttpResponseRedirect(path)

    def delete_translation(self, request, object_id, extra_context=None):
        if 'language' in request.GET:
            language = request.GET['language']
        else:
            language = get_language_from_request(request)

        opts = Article._meta
        titleopts = Title._meta
        app_label = titleopts.app_label
        pluginopts = CMSPlugin._meta

        try:
            obj = self.get_queryset(request).get(pk=unquote(object_id))
        except self.model.DoesNotExist:
            # Don't raise Http404 just yet, because we haven't checked
            # permissions yet. We don't want an unauthenticated user to be able
            # to determine whether a given object exists.
            obj = None

        if not self.has_delete_permission(request, obj):
            return HttpResponseForbidden(force_text(_('You do not have permission to change this article')))

        if obj is None:
            raise Http404(
                _('%(name)s object with primary key %(key)r does not exist.') % {
                    'name': force_text(opts.verbose_name),
                    'key': escape(object_id)
                })

        if not len(list(obj.get_languages())) > 1:
            raise Http404(_('There only exists one translation for this article'))

        titleobj = get_object_or_404(Title, article__id=object_id, language=language)
        saved_plugins = CMSPlugin.objects.filter(placeholder__article__id=object_id, language=language)

        using = router.db_for_read(self.model)
        kwargs = {
            'admin_site': self.admin_site,
            'user': request.user,
            'using': using
        }

        deleted_objects, __, perms_needed = get_deleted_objects(
            [titleobj],
            titleopts,
            **kwargs
        )[:3]
        to_delete_plugins, __, perms_needed_plugins = get_deleted_objects(
            saved_plugins,
            pluginopts,
            **kwargs
        )[:3]

        deleted_objects.append(to_delete_plugins)
        perms_needed = set(list(perms_needed) + list(perms_needed_plugins))

        if request.method == 'POST':
            if perms_needed:
                raise PermissionDenied

            message = _('Title and plugins with language %(language)s was deleted') % {
                'language': force_text(get_language_object(language)['name'])
            }
            self.log_change(request, titleobj, message)
            messages.info(request, message)

            titleobj.delete()
            for p in saved_plugins:
                p.delete()

            public = obj.publisher_public
            if public:
                public.save()

            if not self.has_change_permission(request, None):
                return HttpResponseRedirect(admin_reverse('index'))
            return HttpResponseRedirect(admin_reverse('cms_articles_article_changelist'))

        context = {
            'title': _('Are you sure?'),
            'object_name': force_text(titleopts.verbose_name),
            'object': titleobj,
            'deleted_objects': deleted_objects,
            'perms_lacking': perms_needed,
            'opts': opts,
            'root_path': admin_reverse('index'),
            'app_label': app_label,
        }
        context.update(extra_context or {})
        request.current_app = self.admin_site.name
        return render(request, self.delete_confirmation_template or [
            'admin/%s/%s/delete_confirmation.html' % (app_label, titleopts.object_name.lower()),
            'admin/%s/delete_confirmation.html' % app_label,
            'admin/delete_confirmation.html'
        ], context)

    def preview_article(self, request, object_id, language):
        """Redirecting preview function based on draft_id
        """
        article = get_object_or_404(self.model, id=object_id)
        attrs = '?%s' % get_cms_setting('CMS_TOOLBAR_URL__EDIT_ON')
        attrs += '&language=' + language
        with force_language(language):
            url = article.get_absolute_url(language) + attrs
        return HttpResponseRedirect(url)


admin.site.register(Article, ArticleAdmin)



# -*- coding: utf-8 -*-
from platform import python_version

import django
from classytags.arguments import Argument, MultiValueArgument
from classytags.core import Options, Tag
from classytags.helpers import AsTag
from cms.templatetags.cms_tags import PlaceholderOptions
from cms.utils import get_language_from_request, get_site_id
from cms.utils.moderator import use_draft
from django import template
from django.contrib.sites.models import Site
from django.core.mail import mail_managers
from django.middleware.common import BrokenLinkEmailsMiddleware
from django.utils import six
from django.utils.encoding import force_text
from django.utils.html import escape
from django.utils.translation import ugettext_lazy as _
from menus.base import NavigationNode
from menus.templatetags.menu_tags import ShowBreadcrumb

from ..conf import settings
from ..models import Article
from ..utils.placeholder import validate_placeholder_name

DJANGO_VERSION = django.get_version()
PYTHON_VERSION = python_version()

register = template.Library()


def _get_article_by_untyped_arg(article_lookup, request, site_id):
    """
    The `article_lookup` argument can be of any of the following types:
    - Integer: interpreted as `pk` of the desired article
    - `dict`: a dictionary containing keyword arguments to find the desired article
    (for instance: `{'pk': 1}`)
    - `Article`: you can also pass a Article object directly, in which case there will be no database lookup.
    - `None`: the current article will be used
    """
    if article_lookup is None:
        return request.current_article
    if isinstance(article_lookup, Article):
        if hasattr(request, 'current_article') and request.current_article.pk == article_lookup.pk:
            return request.current_article
        return article_lookup
    if isinstance(article_lookup, six.integer_types):
        article_lookup = {'pk': article_lookup}
    elif not isinstance(article_lookup, dict):
        raise TypeError('The article_lookup argument can be either a Dictionary, Integer, or Article.')
    article_lookup.update({'site': site_id})
    try:
        article = Article.objects.all().get(**article_lookup)
        if request and use_draft(request):
            if article.publisher_is_draft:
                return article
            else:
                return article.publisher_draft
        else:
            if article.publisher_is_draft:
                return article.publisher_public
            else:
                return article
    except Article.DoesNotExist:
        site = Site.objects.get_current()
        subject = _('Article not found on %(domain)s') % {'domain': site.domain}
        body = (
            _("A template tag couldn't find the article with lookup arguments `%(article_lookup)s\n`. "
              "The URL of the request was: http://%(host)s%(path)s") %
            {'article_lookup': repr(article_lookup), 'host': site.domain, 'path': request.path_info}
        )
        if settings.DEBUG:
            raise Article.DoesNotExist(body)
        else:
            if getattr(settings, 'SEND_BROKEN_LINK_EMAILS', False):
                mail_managers(subject, body, fail_silently=True)
            elif 'django.middleware.common.BrokenLinkEmailsMiddleware' in settings.MIDDLEWARE_CLASSES:
                middle = BrokenLinkEmailsMiddleware()
                domain = request.get_host()
                path = request.get_full_path()
                referer = force_text(request.META.get('HTTP_REFERER', ''), errors='replace')
                if not middle.is_ignorable_request(request, path, domain, referer):
                    mail_managers(subject, body, fail_silently=True)
            return None


class ArticlePlaceholder(Tag):
    """
    This template node is used to output article content and
    is also used in the admin to dynamically generate input fields.

    eg: {% article_placeholder "placeholder_name" %}

    {% article_placeholder "footer" or %}
        <a href="/about/">About us</a>
    {% endarticle_placeholder %}

    Keyword arguments:
    name -- the name of the placeholder
    or -- optional argument which if given will make the template tag a block
        tag whose content is shown if the placeholder is empty
    """
    name = 'article_placeholder'
    options = PlaceholderOptions(
        Argument('name', resolve=False),
        MultiValueArgument('extra_bits', required=False, resolve=False),
        blocks=[
            ('endarticle_placeholder', 'nodelist'),
        ],
    )

    def render_tag(self, context, name, extra_bits, nodelist=None):
        validate_placeholder_name(name)

        content_renderer = context.get('cms_articles_content_renderer')

        if not content_renderer:
            return ''

        content = content_renderer.render_article_placeholder(
            slot=name,
            context=context,
            nodelist=nodelist,
        )
        return content

    def get_name(self):
        return self.kwargs['name'].var.value.strip('"').strip("'")


register.tag('article_placeholder', ArticlePlaceholder)


class ArticleAttribute(AsTag):
    """
    This template node is used to output an attribute from a article such
    as its title or slug.

    Synopsis
         {% article_attribute "field-name" %}
         {% article_attribute "field-name" as varname %}
         {% article_attribute "field-name" article_lookup %}
         {% article_attribute "field-name" article_lookup as varname %}

    Example
         {# Output current article's page_title attribute: #}
         {% article_attribute "page_title" %}
         {# Output slug attribute of the article with pk 10: #}
         {% article_attribute "slug" 10 %}
         {# Assign page_title attribute to a variable: #}
         {% article_attribute "page_title" as title %}

    Keyword arguments:
    field-name -- the name of the field to output. Use one of:
    - title
    - description
    - page_title
    - slug
    - meta_description
    - changed_date
    - changed_by
    - image

    article_lookup -- lookup argument for Article, if omitted field-name of current article is returned.
    See _get_article_by_untyped_arg() for detailed information on the allowed types and their interpretation
    for the article_lookup argument.

    varname -- context variable name. Output will be added to template context as this variable.
    This argument is required to follow the 'as' keyword.
    """
    name = 'article_attribute'
    options = Options(
        Argument('name', resolve=False),
        Argument('article_lookup', required=False, default=None),
        'as',
        Argument('varname', required=False, resolve=False)
    )

    valid_attributes = [
        "title",
        "slug",
        "description",
        "page_title",
        "menu_title",
        "meta_description",
        "changed_date",
        "changed_by",
        "image",
    ]

    def get_value(self, context, name, article_lookup):
        if 'request' not in context:
            return ''
        name = name.lower()
        request = context['request']
        lang = get_language_from_request(request)
        article = _get_article_by_untyped_arg(article_lookup, request, get_site_id(None))
        if article and name in self.valid_attributes:
            func = getattr(article, "get_%s" % name)
            ret_val = func(language=lang, fallback=True)
            if name not in ("changed_date", "image"):
                ret_val = escape(ret_val)
            return ret_val
        return ''


register.tag('article_attribute', ArticleAttribute)


class ShowArticleBreadcrumb(ShowBreadcrumb):
    name = 'show_article_breadcrumb'

    def get_context(self, context, start_level, template, only_visible):
        context = super(ShowArticleBreadcrumb, self).get_context(context, start_level, template, only_visible)
        try:
            current_article = context['request'].current_article
        except:
            pass
        else:
            context['ancestors'].append(NavigationNode(
                title=current_article.get_menu_title(),
                url=current_article.get_absolute_url(),
                id=current_article.pk,
                visible=True,
            ))
        return context


register.tag('show_article_breadcrumb', ShowArticleBreadcrumb)


@register.simple_tag(takes_context=True)
def url_page(context, page):
    get = context['request'].GET.copy()
    get[settings.CMS_ARTICLES_PAGE_FIELD] = page
    return '{}?{}'.format(context['request'].path, get.urlencode())



from django.db import models
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _


@python_2_unicode_compatible
class Attribute(models.Model):
    name = models.CharField(_('name'), max_length=255)

    class Meta:
        app_label = 'cms_articles'
        verbose_name = _('attribute')
        verbose_name_plural = _('attributes')

    def __str__(self):
        return force_text(self.name)



from .category import Category
from .article import Article
from .title import Title
from .attribute import Attribute
from .plugins import ArticlePlugin, ArticlesPlugin, ArticlesCategoryPlugin

(Category, Article, Title, Attribute, ArticlePlugin, ArticlesPlugin, ArticlesCategoryPlugin)



from cms.constants import PUBLISHER_STATE_DIRTY
from cms.models import CMSPlugin
from cms.signals.plugins import get_placeholder
from django.db.models import signals


def set_dirty(plugin):
    placeholder = get_placeholder(plugin)

    if placeholder:
        for article in placeholder.cms_articles.all():
            article.title_set.filter(language=plugin.language).update(publisher_state=PUBLISHER_STATE_DIRTY)


def pre_save_plugins(**kwargs):
    plugin = kwargs['instance']

    if hasattr(plugin, '_no_reorder'):
        return

    set_dirty(plugin)

    if not plugin.pk:
        return

    try:
        old_plugin = (
            CMSPlugin
            .objects
            .select_related('placeholder')
            .only('language', 'placeholder')
            .exclude(placeholder=plugin.placeholder_id)
            .get(pk=plugin.pk)
        )
    except CMSPlugin.DoesNotExist:
        pass
    else:
        set_dirty(old_plugin)


def pre_delete_plugins(**kwargs):
    plugin = kwargs['instance']
    if hasattr(plugin, '_no_reorder'):
        return

    set_dirty(plugin)


signals.pre_save.connect(pre_save_plugins, sender=CMSPlugin, dispatch_uid='cms_pre_save_plugin')
signals.pre_delete.connect(pre_delete_plugins, sender=CMSPlugin, dispatch_uid='cms_pre_delete_plugin')



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class CMSArticlesConfig(AppConfig):
    name = 'cms_articles'
    verbose_name = _('django CMS articles')

    def ready(self):
        from . import signals
        signals  # just use it



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-06-22 12:45
from __future__ import unicode_literals

from django.db import migrations, models


def migrate_plugins(apps, schema_editor):
    ArticlesPlugin = apps.get_model('cms_articles', 'ArticlesPlugin')

    for p in ArticlesPlugin.objects.all():
        if p.tree:
            p.trees.add(p.tree)


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0001_initial'),
        ('cms_articles', '0006_order_date'),
    ]

    operations = [
        migrations.AddField(
            model_name='articlesplugin',
            name='trees',
            field=models.ManyToManyField(blank=True, null=True, related_name='_articlesplugin_trees_+', to='cms.Page', verbose_name='trees'),
        ),
        migrations.RunPython(migrate_plugins),
        migrations.RemoveField(
            model_name='articlesplugin',
            name='tree',
        ),
    ]



from cms.models import Page
from django.db import models
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from ..conf import settings


@python_2_unicode_compatible
class Category(models.Model):
    page = models.OneToOneField(
        Page, verbose_name=_('page'), related_name='cms_articles_category',
        limit_choices_to={'publisher_is_draft': True, 'site_id': settings.SITE_ID})

    class Meta:
        app_label = 'cms_articles'
        verbose_name = _('category')
        verbose_name_plural = _('categories')

    def __str__(self):
        return force_text(self.page)



from django.utils.translation import ugettext_lazy as _

# by default, use the same templates as for cms pages
CMS_ARTICLES_TEMPLATES = [
    ('cms_articles/default.html', _('Default'))
]

# default slug format
CMS_ARTICLES_SLUG_FORMAT = '{now:%Y-%m}-{slug}'
CMS_ARTICLES_SLUG_REGEXP = r'[0-9]{4}-[0-9]{2}-([^/]+)'

# templates used to render plugin article
CMS_ARTICLES_PLUGIN_ARTICLE_TEMPLATES = [
    ('default', _('Default')),
]

# templates used to render plugin articles
CMS_ARTICLES_PLUGIN_ARTICLES_TEMPLATES = [
    ('default', _('Default')),
]

# the main slot for initial content to be stored in
CMS_ARTICLES_SLOT = 'content'

CMS_ARTICLES_USE_HAYSTACK = True

CMS_ARTICLES_PAGE_FIELD = 'page'
CMS_ARTICLES_YEAR_FIELD = 'year'
CMS_ARTICLES_MONTH_FIELD = 'month'
CMS_ARTICLES_DAY_FIELD = 'day'



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-13 14:29
from __future__ import unicode_literals

from django.db import migrations, models


def set_order_dates(apps, schema_editor):
    Article = apps.get_model('cms_articles', 'Article')

    for article in Article.objects.all():
        article.order_date = article.publication_date or article.creation_date
        article.save()


class Migration(migrations.Migration):

    dependencies = [
        ('cms_articles', '0005_attributes'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='article',
            options={'ordering': ('-order_date',), 'permissions': (('publish_article', 'Can publish page'),), 'verbose_name': 'article', 'verbose_name_plural': 'articles'},
        ),
        migrations.AddField(
            model_name='article',
            name='order_date',
            field=models.DateTimeField(null=True),
        ),
        migrations.RunPython(set_order_dates),
        migrations.AlterField(
            model_name='article',
            name='order_date',
            field=models.DateTimeField(auto_now_add=True, verbose_name='publication or creation time'),
        ),
    ]



# -*- coding: utf-8 -*-
from cms.toolbar.utils import get_toolbar_from_request

from .plugin_rendering import ArticlesContentRenderer


def cms_articles(request):
    """
    Adds cms-articles-related variables to the context.
    """
    toolbar = get_toolbar_from_request(request)
    return {
        'cms_articles_content_renderer': ArticlesContentRenderer(toolbar.content_renderer),
    }



import warnings

from django.db.models import signals
from django.template import TemplateDoesNotExist

from ..models import Article


def pre_save_article(instance, **kwargs):
    instance.old_article = None
    try:
        instance.old_article = Article.objects.get(pk=instance.pk)
    except Article.DoesNotExist:
        pass


def post_save_article(instance, raw, **kwargs):
    if not raw:
        try:
            instance.rescan_placeholders()
        except TemplateDoesNotExist as e:
            warnings.warn('Exception occurred: %s template does not exists' % e)


def pre_delete_article(instance, **kwargs):
    for placeholder in instance.get_placeholders():
        for plugin in placeholder.cmsplugin_set.all().order_by('-depth'):
            plugin._no_reorder = True
            plugin.delete(no_mp=True)
        placeholder.delete()


signals.pre_save.connect(pre_save_article, sender=Article, dispatch_uid='cms_articles_pre_save_article')
signals.post_save.connect(post_save_article, sender=Article, dispatch_uid='cms_articles_post_save_article')
signals.pre_delete.connect(pre_delete_article, sender=Article, dispatch_uid='cms_articles_pre_delete_article')



from django.db.models import signals

from ..models import Title


def pre_save_title(instance, **kwargs):
    ''' Update article.languages
    '''
    if instance.article.languages:
        languages = instance.article.languages.split(',')
    else:
        languages = []
    if instance.language not in languages:
        languages.append(instance.language)
        instance.article.languages = ','.join(languages)
        instance.article._publisher_keep_state = True
        instance.article.save(no_signals=True)


def pre_delete_title(instance, **kwargs):
    ''' Update article.languages
    '''
    if instance.article.languages:
        languages = instance.article.languages.split(',')
    else:
        languages = []
    if instance.language in languages:
        languages.remove(instance.language)
        instance.article.languages = ','.join(languages)
        instance.article._publisher_keep_state = True
        instance.article.save(no_signals=True)


signals.pre_save.connect(pre_save_title, sender=Title, dispatch_uid='cms_articles_pre_save_article')
signals.pre_delete.connect(pre_delete_title, sender=Title, dispatch_uid='cms_articles_pre_delete_article')



from cms.publisher import PublisherManager
from cms.utils.i18n import get_fallback_languages
from django.contrib.sites.models import Site
from django.db.models import Q

from .query import ArticleQuerySet


class ArticleManager(PublisherManager):
    """Use draft() and public() methods for accessing the corresponding
    instances.
    """

    def get_queryset(self):
        """Change standard model queryset to our own.
        """
        return ArticleQuerySet(self.model)

    def search(self, q, language=None, current_site_only=True):
        """Simple search function

        Plugins can define a 'search_fields' tuple similar to ModelAdmin classes
        """
        from cms.plugin_pool import plugin_pool

        qs = self.get_queryset()
        qs = qs.public()

        if current_site_only:
            site = Site.objects.get_current()
            qs = qs.filter(tree__site=site)

        qt = Q(title_set__title__icontains=q)

        # find 'searchable' plugins and build query
        qp = Q()
        plugins = plugin_pool.get_all_plugins()
        for plugin in plugins:
            cmsplugin = plugin.model
            if not (
                hasattr(cmsplugin, 'search_fields') and
                hasattr(cmsplugin, 'cmsplugin_ptr')
            ):
                continue
            field = cmsplugin.cmsplugin_ptr.field
            related_query_name = field.related_query_name()
            if related_query_name and not related_query_name.startswith('+'):
                for field in cmsplugin.search_fields:
                    qp |= Q(**{
                        'placeholders__cmsplugin__{0}__{1}__icontains'.format(
                            related_query_name,
                            field,
                        ): q})
        if language:
            qt &= Q(title_set__language=language)
            qp &= Q(cmsplugin__language=language)

        qs = qs.filter(qt | qp)

        return qs.distinct()


class TitleManager(PublisherManager):
    def get_title(self, article, language, language_fallback=False):
        """
        Gets the latest content for a particular article and language. Falls back
        to another language if wanted.
        """
        try:
            title = self.get(language=language, article=article)
            return title
        except self.model.DoesNotExist:
            if language_fallback:
                try:
                    titles = self.filter(article=article)
                    fallbacks = get_fallback_languages(language)
                    for lang in fallbacks:
                        for title in titles:
                            if lang == title.language:
                                return title
                    return None
                except self.model.DoesNotExist:
                    pass
            else:
                raise
        return None

    # created new public method to meet test case requirement and to get a list of titles for published articles
    def public(self):
        return self.get_queryset().filter(publisher_is_draft=False, published=True)

    def drafts(self):
        return self.get_queryset().filter(publisher_is_draft=True)

    def set_or_create(self, request, article, form, language):
        """
        set or create a title for a particular article and language
        """
        base_fields = [
            'slug',
            'title',
            'description',
            'meta_description',
            'page_title',
            'menu_title',
            'image',
        ]
        cleaned_data = form.cleaned_data
        try:
            obj = self.get(article=article, language=language)
        except self.model.DoesNotExist:
            data = {}
            for name in base_fields:
                if name in cleaned_data:
                    data[name] = cleaned_data[name]
            data['article'] = article
            data['language'] = language
            return self.create(**data)
        for name in base_fields:
            if name in form.base_fields:
                value = cleaned_data.get(name, None)
                setattr(obj, name, value)
        obj.save()
        return obj



from cms.models import CMSPlugin, Page
from django.core.validators import MinValueValidator
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from ..conf import settings
from .article import Article
from .attribute import Attribute
from .category import Category


@python_2_unicode_compatible
class ArticlePlugin(CMSPlugin):
    article = models.ForeignKey(
        Article, verbose_name=_('article'), related_name='+', limit_choices_to={'publisher_is_draft': True})
    template = models.CharField(
        _('Template'), max_length=100,
        choices=settings.CMS_ARTICLES_PLUGIN_ARTICLE_TEMPLATES,
        default=settings.CMS_ARTICLES_PLUGIN_ARTICLE_TEMPLATES[0][0],
        help_text=_('The template used to render plugin.'))

    def __str__(self):
        return self.article.get_title()

    def get_article(self, context):
        try:
            edit_mode = context['request'].toolbar.edit_mode
        except:
            edit_mode = False

        if edit_mode:
            return self.article
        else:
            return self.article.get_published_object()


class ArticlesPluginBase(CMSPlugin):
    number = models.PositiveSmallIntegerField(_('Number of last articles'), default=3,
                                              validators=[MinValueValidator(1)])
    template = models.CharField(
        _('Template'), max_length=100,
        choices=settings.CMS_ARTICLES_PLUGIN_ARTICLES_TEMPLATES,
        default=settings.CMS_ARTICLES_PLUGIN_ARTICLES_TEMPLATES[0][0],
        help_text=_('The template used to render plugin.'))
    attributes = models.ManyToManyField(Attribute, verbose_name=_('attributes'), related_name='+', blank=True)

    class Meta:
        abstract = True

    def copy_relations(self, oldinstance):
        self.attributes = oldinstance.attributes.all()

    def get_articles(self, context):
        try:
            edit_mode = context['request'].toolbar.edit_mode
        except:
            edit_mode = False

        if edit_mode:
            articles = Article.objects.drafts()
        else:
            articles = Article.objects.public().published()

        if self.attributes.count():
            articles = articles.filter(attributes=self.attributes.all())

        return articles


@python_2_unicode_compatible
class ArticlesPlugin(ArticlesPluginBase):
    trees = models.ManyToManyField(
        Page, verbose_name=_('trees'), related_name='+', blank=True,
        limit_choices_to={
            'publisher_is_draft': False,
            'application_urls': 'CMSArticlesApp',
            'site_id': settings.SITE_ID,
        })
    categories = models.ManyToManyField(Category, verbose_name=_('categories'), related_name='+', blank=True)

    def __str__(self):
        return _('last {} articles').format(self.number)

    def get_articles(self, context):
        articles = super(ArticlesPlugin, self).get_articles(context)

        if self.trees.count():
            articles = articles.filter(tree__in=self.trees.all())

        if self.categories.count():
            articles = articles.filter(categories=self.categories.all())

        return articles

    def copy_relations(self, oldinstance):
        self.trees = oldinstance.trees.all()
        self.categories = oldinstance.categories.all()
        super(ArticlesPlugin, self).copy_relations(oldinstance)


@python_2_unicode_compatible
class ArticlesCategoryPlugin(ArticlesPluginBase):
    subcategories = models.BooleanField(
        _('include sub-categories'), default=False,
        help_text=_('Check, if you want to include articles from sub-categories of this category.'))

    def __str__(self):
        return _('last {} articles in this category').format(self.number)

    def get_articles(self, context):
        # no page - no category
        if self.placeholder.page is None:
            return []

        page = self.placeholder.page.get_draft_object()
        try:
            category = page.cms_articles_category
        except:
            category = Category.objects.create(page=page)

        articles = super(ArticlesPlugin, self).get_articles(context)

        if self.subcategories:
            if not self.placeholder.page.is_home:
                articles = articles.filter(categories__page__in=self.placeholder.page.get_descendants(True))
            # if self.placeholder.page.is_home, take all
        else:
            articles = articles.filter(categories=category)

        return articles



from . import article, plugins, title
from django.dispatch import Signal

{article, plugins, title}

post_publish_article = Signal(providing_args=['instance', 'language'])
post_unpublish_article = Signal(providing_args=['instance', 'language'])



import datetime

from cms.api import add_plugin
from cms.utils.i18n import get_language_list
from cms.utils.permissions import current_user
from django.template.defaultfilters import slugify
from django.template.loader import get_template
from django.utils.encoding import force_text
from django.utils.timezone import now

from djangocms_text_ckeditor.cms_plugins import TextPlugin

from .conf import settings
from .models import Article, Title


"""
Public Python API to create CMS articles.

WARNING: None of the functions defined in this module checks for permissions.
You must implement the necessary permission checks in your own code before
calling these methods!
"""


def create_article(tree, template, title, language, slug=None, description=None,
                   page_title=None, menu_title=None, meta_description=None,
                   created_by=None, image=None, publication_date=None, publication_end_date=None,
                   published=False, login_required=False, creation_date=None, categories=[]):
    """
    Create a CMS Article and it's title for the given language
    """

    # validate tree
    tree = tree.get_public_object()
    assert tree.application_urls == 'CMSArticlesApp'

    # validate template
    assert template in [tpl[0] for tpl in settings.CMS_ARTICLES_TEMPLATES]
    get_template(template)

    # validate language:
    assert language in get_language_list(tree.site_id), settings.CMS_LANGUAGES.get(tree.site_id)

    # validate publication date
    if publication_date:
        assert isinstance(publication_date, datetime.date)

    # validate publication end date
    if publication_end_date:
        assert isinstance(publication_end_date, datetime.date)

    # validate creation date
    if not creation_date:
        creation_date = publication_date
    if creation_date:
        assert isinstance(creation_date, datetime.date)

    # get username
    if created_by:
        try:
            username = created_by.get_username()
        except:
            username = force_text(created_by)
    else:
        username = 'script'

    with current_user(username):
        # create article
        article = Article.objects.create(
            tree=tree,
            template=template,
            login_required=login_required,
            creation_date=creation_date,
            publication_date=publication_date,
            publication_end_date=publication_end_date,
            languages=language,
        )
        for category in categories:
            article.categories.add(category)

        # create title
        create_title(
            article=article,
            language=language,
            title=title,
            slug=slug,
            description=description,
            page_title=page_title,
            menu_title=menu_title,
            meta_description=meta_description,
            creation_date=creation_date,
            image=image,
        )

        # publish article
        if published:
            article.publish(language)

    return article.reload()


def create_title(article, language, title, slug=None, description=None,
                 page_title=None, menu_title=None, meta_description=None,
                 creation_date=None, image=None):
    """
    Create an article title.
    """
    # validate article
    assert isinstance(article, Article)

    # validate language:
    assert language in get_language_list(article.tree.site_id)

    # validate creation date
    if creation_date:
        assert isinstance(creation_date, datetime.date)

    # set default slug:
    if not slug:
        slug = settings.CMS_ARTICLES_SLUG_FORMAT.format(
            now=creation_date or now(),
            slug=slugify(title),
        )

    # find unused slug:
    base_slug = slug
    qs = Title.objects.filter(language=language)
    used_slugs = list(s for s in qs.values_list('slug', flat=True) if s.startswith(base_slug))
    i = 1
    while slug in used_slugs:
        slug = '%s-%s' % (base_slug, i)
        i += 1

    # create title
    title = Title.objects.create(
        article=article,
        language=language,
        title=title,
        slug=slug,
        description=description,
        page_title=page_title,
        menu_title=menu_title,
        meta_description=meta_description,
        image=image,
    )

    return title


def add_content(obj, language, slot, content):
    """
    Adds a TextPlugin with given content to given slot
    """
    placeholder = obj.placeholders.get(slot=slot)
    add_plugin(placeholder, TextPlugin, language, body=content)


def publish_article(article, language, changed_by=None):
    """
    Publish an article. This sets `article.published` to `True`
    and calls article.publish() which does the actual publishing.
    """
    article = article.reload()

    # get username
    if changed_by:
        username = changed_by.get_username()
    else:
        username = 'script'

    with current_user(username):
        article.publish(language)

    return article.reload()



from cms.publisher.query import PublisherQuerySet
from django.db.models import Q
from django.utils import timezone


class ArticleQuerySet(PublisherQuerySet):
    def published(self, language=None):
        qs = self.filter(
            Q(publication_date__lte=timezone.now()) | Q(publication_date__isnull=True),
            Q(publication_end_date__gt=timezone.now()) | Q(publication_end_date__isnull=True),
            title_set__published=True)
        if language:
            return qs.filter(title_set__language=language)
        else:
            return qs



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from setuptools import find_packages, setup

setup(
    name = 'django-cms-articles',
    version = '1.1.3',
    description = 'django CMS application for managing articles',
    author = 'Jakub Dorňák',
    author_email = 'jakub.dornak@misli.cz',
    license = 'BSD',
    url = 'https://github.com/misli/django-cms-articles',
    packages = find_packages(),
    include_package_data = True,
    install_requires=[
        'django-cms>=3.4,<3.5',
        'django-filer',
        'python-dateutil',
    ],
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: Czech',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
    ],
)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-27 12:00
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cms_articles', '0004_categories'),
    ]

    operations = [
        migrations.CreateModel(
            name='Attribute',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='name')),
            ],
            options={
                'verbose_name': 'attribute',
                'verbose_name_plural': 'attributes',
            },
        ),
        migrations.AddField(
            model_name='article',
            name='attributes',
            field=models.ManyToManyField(blank=True, related_name='articles', to='cms_articles.Attribute', verbose_name='attributes'),
        ),
        migrations.AddField(
            model_name='articlescategoryplugin',
            name='attributes',
            field=models.ManyToManyField(blank=True, related_name='_articlescategoryplugin_attributes_+', to='cms_articles.Attribute', verbose_name='attributes'),
        ),
        migrations.AddField(
            model_name='articlesplugin',
            name='attributes',
            field=models.ManyToManyField(blank=True, related_name='_articlesplugin_attributes_+', to='cms_articles.Attribute', verbose_name='attributes'),
        ),
    ]



from django.conf.urls import url

from . import views
from .conf import settings

if settings.APPEND_SLASH:
    regexp = r'^(?P<slug>{})/$'.format(settings.CMS_ARTICLES_SLUG_REGEXP)
else:
    regexp = r'^(?P<slug>{})$'.format(settings.CMS_ARTICLES_SLUG_REGEXP)

urlpatterns = [
    url(regexp, views.article, name='article'),
]



from django import forms
from django.utils.translation import ugettext_lazy as _

from .models import Item, Options
from .utils import import_wordpress


class XMLImportForm(forms.ModelForm):
    wordpress_xml = forms.FileField(
        label=_('WordPress XML file'),
        help_text=_('Select XML file with posts exported from WP.')
    )

    class Meta:
        model = Item
        fields = []

    def save(self, *args, **kwargs):
        return import_wordpress(self.cleaned_data['wordpress_xml'])


class CMSImportForm(forms.Form):
    options = forms.ModelChoiceField(
        label=_('Options'),
        queryset=Options.objects.order_by('name'),
    )



# -*- coding: utf-8 -*-
from django.conf.urls import url
from django.contrib import admin, messages
from django.contrib.admin import helpers
from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.db import transaction
from django.http import HttpResponse, HttpResponseBadRequest
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _

from .forms import CMSImportForm, XMLImportForm
from .models import Author, Category, Item, Options

User = get_user_model()


class AuthorAdmin(admin.ModelAdmin):
    search_fields = ['login', 'email', 'first_name', 'last_name']
    list_display = ['login', 'email', 'first_name', 'last_name', 'user']
    list_editable = ['user']
    actions = ['create_users', 'find_users']

    def has_add_permission(self, request):
        return False

    def create_users(self, request, queryset):
        for author in queryset.all():
            if author.user:
                continue
            try:
                author.user = User.objects.create(
                    username=author.login,
                    email=author.email,
                    first_name=author.first_name or '',
                    last_name=author.last_name or '',
                )
                author.save()
            except Exception as e:
                self.message_user(request, _('Failed to create user {}: {}').format(author.login, e), messages.ERROR)
            else:
                self.message_user(request, _('Successfully created user {}').format(author.login), messages.SUCCESS)
    create_users.short_description = _('Create users for selected authors')

    def find_users(self, request, queryset):
        for author in queryset.all():
            if author.user:
                continue

            try:
                author.user = User.objects.get(username=author.login)
            except User.DoesNotExist:
                pass

            if not author.user:
                author.user = User.objects.filter(email=author.email).first()

            if not author.user:
                author.user = User.objects.filter(first_name=author.first_name, last_name=author.last_name).first()

            if author.user:
                author.save()
                self.message_user(
                    request,
                    _('Successfully found user {} for author').format(author.user, author),
                    messages.SUCCESS)
            else:
                self.message_user(request, _('Failed to find user for author {}').format(author), messages.ERROR)
    find_users.short_description = _('Find users for selected authors')


admin.site.register(Author, AuthorAdmin)


class CategoryAdmin(admin.ModelAdmin):
    search_fields = ['=term_id', 'name', 'slug']
    list_display = ['slug', 'cached_name', 'category']
    list_editable = ['category']
    ordering = ['cached_name']

    def has_add_permission(self, request):
        return False


admin.site.register(Category, CategoryAdmin)


class ItemAdmin(admin.ModelAdmin):
    search_fields = ['=post_id', '=post_parent', 'categories__name', 'title']
    list_filter = ['post_type', 'status', 'categories']
    list_display = ['post_id', 'parent_link', 'children_link', 'title_link',
                    'post_type', 'post_date', 'status', 'imported_link']
    actions = ['cms_import']

    def get_urls(self):
        return [
            url(r'^import/$', self.import_item, name="{}_{}_import_item".format(
                self.model._meta.app_label, self.model._meta.model_name,
            )),
        ] + super(ItemAdmin, self).get_urls()

    def save_model(self, request, obj, form, change):
        pass

    def save_related(self, request, form, formsets, change):
        pass

    def log_addition(self, request, object, message):
        pass

    def response_add(self, request, obj, post_url_continue=None):
        for error in obj['errors']:
            self.message_user(request, error, messages.ERROR)
        for msg in (
            _('Successfullty imported {} authors').format(obj['authors']),
            _('Successfullty imported {} categories').format(obj['categories']),
            _('Successfullty imported {} items').format(obj['items']),
        ):
            self.message_user(request, msg, messages.SUCCESS)
        return self.response_post_save_add(request, obj)

    def get_form(self, request, obj=None, **kwargs):
        if not obj:
            kwargs['form'] = XMLImportForm
        return super(ItemAdmin, self).get_form(request, obj, **kwargs)

    def parent_link(self, obj):
        if obj.post_parent:
            return '<a href="{url}">{label}</a>'.format(
                url=reverse('admin:{}_{}_changelist'.format(
                    Item._meta.app_label,
                    Item._meta.model_name,
                )) + '?post_id__exact={}'.format(obj.post_parent),
                label=obj.post_parent,
            )
        else:
            return ''
    parent_link.short_description = _('post parent')
    parent_link.admin_order_field = 'post_parent'
    parent_link.allow_tags = True

    def children_link(self, obj):
        count = obj.children.count()
        if count:
            return '<a href="{url}">{label}</a>'.format(
                url=reverse('admin:{}_{}_changelist'.format(
                    Item._meta.app_label,
                    Item._meta.model_name,
                )) + '?post_parent__exact={}'.format(obj.post_id),
                label=count,
            )
        else:
            return ''
    children_link.short_description = _('post children')
    children_link.allow_tags = True

    def title_link(self, obj):
        return '<a href="{url}" title="{url}" target="_blank">{title}</a>'.format(
            url=obj.guid,
            title=obj.title,
        )
    title_link.short_description = _('title')
    title_link.admin_order_field = 'title'
    title_link.allow_tags = True

    def imported_link(self, obj):
        url = None
        if obj.article or obj.page:
            url = (obj.article or obj.page).get_absolute_url()
        elif obj.file:
            url = obj.file.file.url
        elif obj.folder:
            url = obj.folder.get_admin_directory_listing_url_path()
        if url:
            return '<a href="{url}" target="_blank">{obj}</a>'.format(
                obj=obj.article or obj.page or obj.file or obj.folder,
                url=url,
            )
        else:
            return ''
    imported_link.short_description = _('imported as')
    imported_link.allow_tags = True

    def cms_import(self, request, queryset):
        if request.POST.get('post', 'no') == 'yes':
            form = CMSImportForm(request.POST)
            if form.is_valid():
                return render_to_response('cms_articles/import_wordpress/cms_import.html', {
                    'title': _('Running import'),
                    'items': queryset,
                    'options': form.cleaned_data['options'],
                    'media': self.media,
                    'opts': self.model._meta,
                }, context_instance=RequestContext(request))
        else:
            form = CMSImportForm()
        return render_to_response('cms_articles/import_wordpress/form.html', {
            'title': _('Select predefined import options'),
            'queryset': queryset,
            'opts': self.model._meta,
            'form': form,
            'action_checkbox_name': helpers.ACTION_CHECKBOX_NAME,
        }, context_instance=RequestContext(request))
    cms_import.short_description = _('Import selected items into CMS')

    @transaction.atomic
    def import_item(self, request):
        try:
            item_id = int(request.GET['item_id'])
            options_id = int(request.GET['options_id'])
        except:
            return HttpResponseBadRequest()
        item = get_object_or_404(Item, id=item_id)
        options = get_object_or_404(Options, id=options_id)
        item.cms_import(options)
        return HttpResponse('0', content_type="text/json")


admin.site.register(Item, ItemAdmin)


class OptionsAdmin(admin.ModelAdmin):
    search_fields = ['name']
    save_as = True

    fieldsets = [
        (None, {'fields': ['name']}),
        (_('Global options'), {'fields': ['language']}),
        (_('Article specific options'), {'fields': ['article_tree', 'article_template', 'article_slot',
                                                    'article_folder', 'article_redirects', 'article_publish']}),
        (_('Page specific options'), {'fields': ['page_root', 'page_template', 'page_slot',
                                                 'page_folder', 'page_redirects', 'page_publish']}),
        (_('File specific options'), {'fields': ['file_folder']}),
        (_('Gallery specific options'), {'fields': ['gallery_folder']}),
        (_('Slide specific options'), {'fields': ['slide_folder']}),
    ]


admin.site.register(Options, OptionsAdmin)



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class CmsArticlesImportWordpressConfig(AppConfig):
    name = 'cms_articles.import_wordpress'
    verbose_name = _('django CMS articles - import from WordPress')
